#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Generated Mon Aug 25 00:38:10 2014 by generateDS.py version 2.12e.
#
# Command line options:
#   ('--external-encoding', 'utf-8')
#   ('--no-process-includes', '')
#   ('-o', 'ebas_generated.py')
#
# Command line arguments:
#   schema/ebas.xsd
#
# Command line:
#   /Library/Frameworks/Python.framework/Versions/2.7/bin/generateDS.py --external-encoding="utf-8" --no-process-includes -o "ebas_generated.py" schema/ebas.xsd
#
# Current working directory (os.getcwd()):
#   generated
#

import sys
import getopt
import re as re_
import base64
import datetime as datetime_
from six import string_types

etree_ = None
Verbose_import_ = False
(
    XMLParser_import_none, XMLParser_import_lxml,
    XMLParser_import_elementtree
) = list(range(3))
XMLParser_import_library = None
try:
    # lxml
    from lxml import etree as etree_
    XMLParser_import_library = XMLParser_import_lxml
    if Verbose_import_:
        print("running with lxml.etree")
except ImportError:
    try:
        # cElementTree from Python 2.5+
        import xml.etree.cElementTree as etree_
        XMLParser_import_library = XMLParser_import_elementtree
        if Verbose_import_:
            print("running with cElementTree on Python 2.5+")
    except ImportError:
        try:
            # ElementTree from Python 2.5+
            import xml.etree.ElementTree as etree_
            XMLParser_import_library = XMLParser_import_elementtree
            if Verbose_import_:
                print("running with ElementTree on Python 2.5+")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as etree_
                XMLParser_import_library = XMLParser_import_elementtree
                if Verbose_import_:
                    print("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as etree_
                    XMLParser_import_library = XMLParser_import_elementtree
                    if Verbose_import_:
                        print("running with ElementTree")
                except ImportError:
                    raise ImportError(
                        "Failed to import ElementTree from any known place")


def parsexml_(*args, **kwargs):
    if (XMLParser_import_library == XMLParser_import_lxml and
            'parser' not in kwargs):
        # Use the lxml ElementTree compatible parser so that, e.g.,
        #   we ignore comments.
        kwargs['parser'] = etree_.ETCompatXMLParser()
    doc = etree_.parse(*args, **kwargs)
    return doc

#
# User methods
#
# Calls to the methods in these classes are generated by generateDS.py.
# You can replace these methods by re-implementing the following class
#   in a module named generatedssuper.py.

try:
    from generatedssuper import GeneratedsSuper
except ImportError as exp:

    class GeneratedsSuper(object):
        tzoff_pattern = re_.compile(r'(\+|-)((0\d|1[0-3]):[0-5]\d|14:00)$')
        class _FixedOffsetTZ(datetime_.tzinfo):
            def __init__(self, offset, name):
                self.__offset = datetime_.timedelta(minutes=offset)
                self.__name = name
            def utcoffset(self, dt):
                return self.__offset
            def tzname(self, dt):
                return self.__name
            def dst(self, dt):
                return None
        def gds_format_string(self, input_data, input_name=''):
            return input_data
        def gds_validate_string(self, input_data, node, input_name=''):
            if not input_data:
                return ''
            else:
                return input_data
        def gds_format_base64(self, input_data, input_name=''):
            return base64.b64encode(input_data)
        def gds_validate_base64(self, input_data, node, input_name=''):
            return input_data
        def gds_format_integer(self, input_data, input_name=''):
            return '%d' % input_data
        def gds_validate_integer(self, input_data, node, input_name=''):
            return input_data
        def gds_format_integer_list(self, input_data, input_name=''):
            return '%s' % input_data
        def gds_validate_integer_list(self, input_data, node, input_name=''):
            values = input_data.split()
            for value in values:
                try:
                    float(value)
                except (TypeError, ValueError):
                    raise_parse_error(node, 'Requires sequence of integers')
            return input_data
        def gds_format_float(self, input_data, input_name=''):
            return ('%.15f' % input_data).rstrip('0')
        def gds_validate_float(self, input_data, node, input_name=''):
            return input_data
        def gds_format_float_list(self, input_data, input_name=''):
            return '%s' % input_data
        def gds_validate_float_list(self, input_data, node, input_name=''):
            values = input_data.split()
            for value in values:
                try:
                    float(value)
                except (TypeError, ValueError):
                    raise_parse_error(node, 'Requires sequence of floats')
            return input_data
        def gds_format_double(self, input_data, input_name=''):
            return '%e' % input_data
        def gds_validate_double(self, input_data, node, input_name=''):
            return input_data
        def gds_format_double_list(self, input_data, input_name=''):
            return '%s' % input_data
        def gds_validate_double_list(self, input_data, node, input_name=''):
            values = input_data.split()
            for value in values:
                try:
                    float(value)
                except (TypeError, ValueError):
                    raise_parse_error(node, 'Requires sequence of doubles')
            return input_data
        def gds_format_boolean(self, input_data, input_name=''):
            return ('%s' % input_data).lower()
        def gds_validate_boolean(self, input_data, node, input_name=''):
            return input_data
        def gds_format_boolean_list(self, input_data, input_name=''):
            return '%s' % input_data
        def gds_validate_boolean_list(self, input_data, node, input_name=''):
            values = input_data.split()
            for value in values:
                if value not in ('true', '1', 'false', '0', ):
                    raise_parse_error(
                        node,
                        'Requires sequence of booleans '
                        '("true", "1", "false", "0")')
            return input_data
        def gds_validate_datetime(self, input_data, node, input_name=''):
            return input_data
        def gds_format_datetime(self, input_data, input_name=''):
            if input_data.microsecond == 0:
                _svalue = '%04d-%02d-%02dT%02d:%02d:%02d' % (
                    input_data.year,
                    input_data.month,
                    input_data.day,
                    input_data.hour,
                    input_data.minute,
                    input_data.second,
                )
            else:
                _svalue = '%04d-%02d-%02dT%02d:%02d:%02d.%s' % (
                    input_data.year,
                    input_data.month,
                    input_data.day,
                    input_data.hour,
                    input_data.minute,
                    input_data.second,
                    ('%f' % (float(input_data.microsecond) / 1000000))[2:],
                )
            if input_data.tzinfo is not None:
                tzoff = input_data.tzinfo.utcoffset(input_data)
                if tzoff is not None:
                    total_seconds = tzoff.seconds + (86400 * tzoff.days)
                    if total_seconds == 0:
                        _svalue += 'Z'
                    else:
                        if total_seconds < 0:
                            _svalue += '-'
                            total_seconds *= -1
                        else:
                            _svalue += '+'
                        hours = total_seconds // 3600
                        minutes = (total_seconds - (hours * 3600)) // 60
                        _svalue += '{0:02d}:{1:02d}'.format(hours, minutes)
            return _svalue
        @classmethod
        def gds_parse_datetime(cls, input_data):
            tz = None
            if input_data[-1] == 'Z':
                tz = GeneratedsSuper._FixedOffsetTZ(0, 'UTC')
                input_data = input_data[:-1]
            else:
                results = GeneratedsSuper.tzoff_pattern.search(input_data)
                if results is not None:
                    tzoff_parts = results.group(2).split(':')
                    tzoff = int(tzoff_parts[0]) * 60 + int(tzoff_parts[1])
                    if results.group(1) == '-':
                        tzoff *= -1
                    tz = GeneratedsSuper._FixedOffsetTZ(
                        tzoff, results.group(0))
                    input_data = input_data[:-6]
            time_parts = input_data.split('.')
            if len(time_parts) > 1:
                micro_seconds = int(float('0.' + time_parts[1]) * 1000000)
                input_data = '%s.%s' % (time_parts[0], micro_seconds, )
                dt = datetime_.datetime.strptime(
                    input_data, '%Y-%m-%dT%H:%M:%S.%f')
            else:
                dt = datetime_.datetime.strptime(
                    input_data, '%Y-%m-%dT%H:%M:%S')
            dt = dt.replace(tzinfo=tz)
            return dt
        def gds_validate_date(self, input_data, node, input_name=''):
            return input_data
        def gds_format_date(self, input_data, input_name=''):
            _svalue = '%04d-%02d-%02d' % (
                input_data.year,
                input_data.month,
                input_data.day,
            )
            try:
                if input_data.tzinfo is not None:
                    tzoff = input_data.tzinfo.utcoffset(input_data)
                    if tzoff is not None:
                        total_seconds = tzoff.seconds + (86400 * tzoff.days)
                        if total_seconds == 0:
                            _svalue += 'Z'
                        else:
                            if total_seconds < 0:
                                _svalue += '-'
                                total_seconds *= -1
                            else:
                                _svalue += '+'
                            hours = total_seconds // 3600
                            minutes = (total_seconds - (hours * 3600)) // 60
                            _svalue += '{0:02d}:{1:02d}'.format(hours, minutes)
            except AttributeError:
                pass
            return _svalue
        @classmethod
        def gds_parse_date(cls, input_data):
            tz = None
            if input_data[-1] == 'Z':
                tz = GeneratedsSuper._FixedOffsetTZ(0, 'UTC')
                input_data = input_data[:-1]
            else:
                results = GeneratedsSuper.tzoff_pattern.search(input_data)
                if results is not None:
                    tzoff_parts = results.group(2).split(':')
                    tzoff = int(tzoff_parts[0]) * 60 + int(tzoff_parts[1])
                    if results.group(1) == '-':
                        tzoff *= -1
                    tz = GeneratedsSuper._FixedOffsetTZ(
                        tzoff, results.group(0))
                    input_data = input_data[:-6]
            dt = datetime_.datetime.strptime(input_data, '%Y-%m-%d')
            dt = dt.replace(tzinfo=tz)
            return dt.date()
        def gds_validate_time(self, input_data, node, input_name=''):
            return input_data
        def gds_format_time(self, input_data, input_name=''):
            if input_data.microsecond == 0:
                _svalue = '%02d:%02d:%02d' % (
                    input_data.hour,
                    input_data.minute,
                    input_data.second,
                )
            else:
                _svalue = '%02d:%02d:%02d.%s' % (
                    input_data.hour,
                    input_data.minute,
                    input_data.second,
                    ('%f' % (float(input_data.microsecond) / 1000000))[2:],
                )
            if input_data.tzinfo is not None:
                tzoff = input_data.tzinfo.utcoffset(input_data)
                if tzoff is not None:
                    total_seconds = tzoff.seconds + (86400 * tzoff.days)
                    if total_seconds == 0:
                        _svalue += 'Z'
                    else:
                        if total_seconds < 0:
                            _svalue += '-'
                            total_seconds *= -1
                        else:
                            _svalue += '+'
                        hours = total_seconds // 3600
                        minutes = (total_seconds - (hours * 3600)) // 60
                        _svalue += '{0:02d}:{1:02d}'.format(hours, minutes)
            return _svalue
        @classmethod
        def gds_parse_time(cls, input_data):
            tz = None
            if input_data[-1] == 'Z':
                tz = GeneratedsSuper._FixedOffsetTZ(0, 'UTC')
                input_data = input_data[:-1]
            else:
                results = GeneratedsSuper.tzoff_pattern.search(input_data)
                if results is not None:
                    tzoff_parts = results.group(2).split(':')
                    tzoff = int(tzoff_parts[0]) * 60 + int(tzoff_parts[1])
                    if results.group(1) == '-':
                        tzoff *= -1
                    tz = GeneratedsSuper._FixedOffsetTZ(
                        tzoff, results.group(0))
                    input_data = input_data[:-6]
            if len(input_data.split('.')) > 1:
                dt = datetime_.datetime.strptime(input_data, '%H:%M:%S.%f')
            else:
                dt = datetime_.datetime.strptime(input_data, '%H:%M:%S')
            dt = dt.replace(tzinfo=tz)
            return dt.time()
        def gds_str_lower(self, instring):
            return instring.lower()
        def get_path_(self, node):
            path_list = []
            self.get_path_list_(node, path_list)
            path_list.reverse()
            path = '/'.join(path_list)
            return path
        Tag_strip_pattern_ = re_.compile(r'\{.*\}')
        def get_path_list_(self, node, path_list):
            if node is None:
                return
            tag = GeneratedsSuper.Tag_strip_pattern_.sub('', node.tag)
            if tag:
                path_list.append(tag)
            self.get_path_list_(node.getparent(), path_list)
        def get_class_obj_(self, node, default_class=None):
            class_obj1 = default_class
            if 'xsi' in node.nsmap:
                classname = node.get('{%s}type' % node.nsmap['xsi'])
                if classname is not None:
                    names = classname.split(':')
                    if len(names) == 2:
                        classname = names[1]
                    class_obj2 = globals().get(classname)
                    if class_obj2 is not None:
                        class_obj1 = class_obj2
            return class_obj1
        def gds_build_any(self, node, type_name=None):
            return None
        @classmethod
        def gds_reverse_node_mapping(cls, mapping):
            return dict(((v, k) for k, v in mapping.items()))


#
# If you have installed IPython you can uncomment and use the following.
# IPython is available from http://ipython.scipy.org/.
#

## from IPython.Shell import IPShellEmbed
## args = ''
## ipshell = IPShellEmbed(args,
##     banner = 'Dropping into IPython',
##     exit_msg = 'Leaving Interpreter, back to program.')

# Then use the following line where and when you want to drop into the
# IPython shell:
#    ipshell('<some message> -- Entering ipshell.\nHit Ctrl-D to exit')

#
# Globals
#

ExternalEncoding = 'utf-8'
Tag_pattern_ = re_.compile(r'({.*})?(.*)')
String_cleanup_pat_ = re_.compile(r"[\n\r\s]+")
Namespace_extract_pat_ = re_.compile(r'{(.*)}(.*)')

#
# Support/utility functions.
#


def showIndent(outfile, level, pretty_print=True):
    if pretty_print:
        for idx in range(level):
            outfile.write('    ')


def quote_xml(inStr):
    if not inStr:
        return ''
    s1 = (isinstance(inStr, string_types) and inStr or
          '%s' % inStr)
    s1 = s1.replace('&', '&amp;')
    s1 = s1.replace('<', '&lt;')
    s1 = s1.replace('>', '&gt;')
    return s1


def quote_attrib(inStr):
    s1 = (isinstance(inStr, string_types) and inStr or
          '%s' % inStr)
    s1 = s1.replace('&', '&amp;')
    s1 = s1.replace('<', '&lt;')
    s1 = s1.replace('>', '&gt;')
    if '"' in s1:
        if "'" in s1:
            s1 = '"%s"' % s1.replace('"', "&quot;")
        else:
            s1 = "'%s'" % s1
    else:
        s1 = '"%s"' % s1
    return s1


def quote_python(inStr):
    s1 = inStr
    if s1.find("'") == -1:
        if s1.find('\n') == -1:
            return "'%s'" % s1
        else:
            return "'''%s'''" % s1
    else:
        if s1.find('"') != -1:
            s1 = s1.replace('"', '\\"')
        if s1.find('\n') == -1:
            return '"%s"' % s1
        else:
            return '"""%s"""' % s1


def get_all_text_(node):
    if node.text is not None:
        text = node.text
    else:
        text = ''
    for child in node:
        if child.tail is not None:
            text += child.tail
    return text


def find_attr_value_(attr_name, node):
    attrs = node.attrib
    attr_parts = attr_name.split(':')
    value = None
    if len(attr_parts) == 1:
        value = attrs.get(attr_name)
    elif len(attr_parts) == 2:
        prefix, name = attr_parts
        namespace = node.nsmap.get(prefix)
        if namespace is not None:
            value = attrs.get('{%s}%s' % (namespace, name, ))
    return value


class GDSParseError(Exception):
    pass


def raise_parse_error(node, msg):
    if XMLParser_import_library == XMLParser_import_lxml:
        msg = '%s (element %s/line %d)' % (
            msg, node.tag, node.sourceline, )
    else:
        msg = '%s (element %s)' % (msg, node.tag, )
    raise GDSParseError(msg)


class MixedContainer:
    # Constants for category:
    CategoryNone = 0
    CategoryText = 1
    CategorySimple = 2
    CategoryComplex = 3
    # Constants for content_type:
    TypeNone = 0
    TypeText = 1
    TypeString = 2
    TypeInteger = 3
    TypeFloat = 4
    TypeDecimal = 5
    TypeDouble = 6
    TypeBoolean = 7
    TypeBase64 = 8
    def __init__(self, category, content_type, name, value):
        self.category = category
        self.content_type = content_type
        self.name = name
        self.value = value
    def getCategory(self):
        return self.category
    def getContenttype(self, content_type):
        return self.content_type
    def getValue(self):
        return self.value
    def getName(self):
        return self.name
    def export(self, outfile, level, name, namespace, pretty_print=True):
        if self.category == MixedContainer.CategoryText:
            # Prevent exporting empty content as empty lines.
            if self.value.strip():
                outfile.write(self.value)
        elif self.category == MixedContainer.CategorySimple:
            self.exportSimple(outfile, level, name)
        else:    # category == MixedContainer.CategoryComplex
            self.value.export(outfile, level, namespace, name, pretty_print)
    def exportSimple(self, outfile, level, name):
        if self.content_type == MixedContainer.TypeString:
            outfile.write('<%s>%s</%s>' % (
                self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeInteger or \
                self.content_type == MixedContainer.TypeBoolean:
            outfile.write('<%s>%d</%s>' % (
                self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeFloat or \
                self.content_type == MixedContainer.TypeDecimal:
            outfile.write('<%s>%f</%s>' % (
                self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeDouble:
            outfile.write('<%s>%g</%s>' % (
                self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeBase64:
            outfile.write('<%s>%s</%s>' % (
                self.name, base64.b64encode(self.value), self.name))
    def to_etree(self, element):
        if self.category == MixedContainer.CategoryText:
            # Prevent exporting empty content as empty lines.
            if self.value.strip():
                if len(element) > 0:
                    if element[-1].tail is None:
                        element[-1].tail = self.value
                    else:
                        element[-1].tail += self.value
                else:
                    if element.text is None:
                        element.text = self.value
                    else:
                        element.text += self.value
        elif self.category == MixedContainer.CategorySimple:
            subelement = etree_.SubElement(element, '%s' % self.name)
            subelement.text = self.to_etree_simple()
        else:    # category == MixedContainer.CategoryComplex
            self.value.to_etree(element)
    def to_etree_simple(self):
        if self.content_type == MixedContainer.TypeString:
            text = self.value
        elif (self.content_type == MixedContainer.TypeInteger or
                self.content_type == MixedContainer.TypeBoolean):
            text = '%d' % self.value
        elif (self.content_type == MixedContainer.TypeFloat or
                self.content_type == MixedContainer.TypeDecimal):
            text = '%f' % self.value
        elif self.content_type == MixedContainer.TypeDouble:
            text = '%g' % self.value
        elif self.content_type == MixedContainer.TypeBase64:
            text = '%s' % base64.b64encode(self.value)
        return text
    def exportLiteral(self, outfile, level, name):
        if self.category == MixedContainer.CategoryText:
            showIndent(outfile, level)
            outfile.write(
                'model_.MixedContainer(%d, %d, "%s", "%s"),\n' % (
                    self.category, self.content_type, self.name, self.value))
        elif self.category == MixedContainer.CategorySimple:
            showIndent(outfile, level)
            outfile.write(
                'model_.MixedContainer(%d, %d, "%s", "%s"),\n' % (
                    self.category, self.content_type, self.name, self.value))
        else:    # category == MixedContainer.CategoryComplex
            showIndent(outfile, level)
            outfile.write(
                'model_.MixedContainer(%d, %d, "%s",\n' % (
                    self.category, self.content_type, self.name,))
            self.value.exportLiteral(outfile, level + 1)
            showIndent(outfile, level)
            outfile.write(')\n')


class MemberSpec_(object):
    def __init__(self, name='', data_type='', container=0):
        self.name = name
        self.data_type = data_type
        self.container = container
    def set_name(self, name): self.name = name
    def get_name(self): return self.name
    def set_data_type(self, data_type): self.data_type = data_type
    def get_data_type_chain(self): return self.data_type
    def get_data_type(self):
        if isinstance(self.data_type, list):
            if len(self.data_type) > 0:
                return self.data_type[-1]
            else:
                return 'xs:string'
        else:
            return self.data_type
    def set_container(self, container): self.container = container
    def get_container(self): return self.container


def _cast(typ, value):
    if typ is None or value is None:
        return value
    return typ(value)

#
# Data representation classes.
#


class Result(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, ResultSet=None):
        self.original_tagname_ = None
        if ResultSet is None:
            self.ResultSet = []
        else:
            self.ResultSet = ResultSet
    def factory(*args_, **kwargs_):
        if Result.subclass:
            return Result.subclass(*args_, **kwargs_)
        else:
            return Result(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_ResultSet(self): return self.ResultSet
    def set_ResultSet(self, ResultSet): self.ResultSet = ResultSet
    def add_ResultSet(self, value): self.ResultSet.append(value)
    def insert_ResultSet_at(self, index, value): self.ResultSet.insert(index, value)
    def replace_ResultSet_at(self, index, value): self.ResultSet[index] = value
    def hasContent_(self):
        if (
            self.ResultSet
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='Result', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Result')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='Result', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='Result'):
        pass
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='Result', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for ResultSet_ in self.ResultSet:
            ResultSet_.export(outfile, level, namespace_, name_='ResultSet', pretty_print=pretty_print)
    def exportLiteral(self, outfile, level, name_='Result'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        showIndent(outfile, level)
        outfile.write('ResultSet=[\n')
        level += 1
        for ResultSet_ in self.ResultSet:
            showIndent(outfile, level)
            outfile.write('model_.ResultSetType(\n')
            ResultSet_.exportLiteral(outfile, level, name_='ResultSetType')
            showIndent(outfile, level)
            outfile.write('),\n')
        level -= 1
        showIndent(outfile, level)
        outfile.write('],\n')
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'ResultSet':
            obj_ = ResultSetType.factory()
            obj_.build(child_)
            self.ResultSet.append(obj_)
            obj_.original_tagname_ = 'ResultSet'
# end class Result


class ResultSetType(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, TimeDimension=None, DataSet=None):
        self.original_tagname_ = None
        self.TimeDimension = TimeDimension
        if DataSet is None:
            self.DataSet = []
        else:
            self.DataSet = DataSet
    def factory(*args_, **kwargs_):
        if ResultSetType.subclass:
            return ResultSetType.subclass(*args_, **kwargs_)
        else:
            return ResultSetType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_TimeDimension(self): return self.TimeDimension
    def set_TimeDimension(self, TimeDimension): self.TimeDimension = TimeDimension
    def get_DataSet(self): return self.DataSet
    def set_DataSet(self, DataSet): self.DataSet = DataSet
    def add_DataSet(self, value): self.DataSet.append(value)
    def insert_DataSet_at(self, index, value): self.DataSet.insert(index, value)
    def replace_DataSet_at(self, index, value): self.DataSet[index] = value
    def hasContent_(self):
        if (
            self.TimeDimension is not None or
            self.DataSet
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='ResultSetType', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ResultSetType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='ResultSetType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='ResultSetType'):
        pass
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='ResultSetType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.TimeDimension is not None:
            self.TimeDimension.export(outfile, level, namespace_, name_='TimeDimension', pretty_print=pretty_print)
        for DataSet_ in self.DataSet:
            DataSet_.export(outfile, level, namespace_, name_='DataSet', pretty_print=pretty_print)
    def exportLiteral(self, outfile, level, name_='ResultSetType'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        if self.TimeDimension is not None:
            showIndent(outfile, level)
            outfile.write('TimeDimension=model_.TimeDimensionType(\n')
            self.TimeDimension.exportLiteral(outfile, level, name_='TimeDimension')
            showIndent(outfile, level)
            outfile.write('),\n')
        showIndent(outfile, level)
        outfile.write('DataSet=[\n')
        level += 1
        for DataSet_ in self.DataSet:
            showIndent(outfile, level)
            outfile.write('model_.DataSetType(\n')
            DataSet_.exportLiteral(outfile, level, name_='DataSetType')
            showIndent(outfile, level)
            outfile.write('),\n')
        level -= 1
        showIndent(outfile, level)
        outfile.write('],\n')
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'TimeDimension':
            obj_ = TimeDimensionType.factory()
            obj_.build(child_)
            self.TimeDimension = obj_
            obj_.original_tagname_ = 'TimeDimension'
        elif nodeName_ == 'DataSet':
            obj_ = DataSetType.factory()
            obj_.build(child_)
            self.DataSet.append(obj_)
            obj_.original_tagname_ = 'DataSet'
# end class ResultSetType


class TimeDimensionType(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, StartTimes=None, EndTimes=None):
        self.original_tagname_ = None
        if isinstance(StartTimes, string_types):
            initvalue_ = datetime_.datetime.strptime(StartTimes, '%Y-%m-%dT%H:%M:%S')
        else:
            initvalue_ = StartTimes
        self.StartTimes = initvalue_
        if isinstance(EndTimes, string_types):
            initvalue_ = datetime_.datetime.strptime(EndTimes, '%Y-%m-%dT%H:%M:%S')
        else:
            initvalue_ = EndTimes
        self.EndTimes = initvalue_
    def factory(*args_, **kwargs_):
        if TimeDimensionType.subclass:
            return TimeDimensionType.subclass(*args_, **kwargs_)
        else:
            return TimeDimensionType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_StartTimes(self): return self.StartTimes
    def set_StartTimes(self, StartTimes): self.StartTimes = StartTimes
    def get_EndTimes(self): return self.EndTimes
    def set_EndTimes(self, EndTimes): self.EndTimes = EndTimes
    def validate_DateTimeListType(self, value):
        # Validate type DateTimeListType, a restriction on xs:dateTime.
        pass
    def hasContent_(self):
        if (
            self.StartTimes is not None or
            self.EndTimes is not None
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='TimeDimensionType', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='TimeDimensionType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='TimeDimensionType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='TimeDimensionType'):
        pass
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='TimeDimensionType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.StartTimes is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sStartTimes>%s</%sStartTimes>%s' % (namespace_, self.gds_format_datetime(self.StartTimes, input_name='StartTimes'), namespace_, eol_))
        if self.EndTimes is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sEndTimes>%s</%sEndTimes>%s' % (namespace_, self.gds_format_datetime(self.EndTimes, input_name='EndTimes'), namespace_, eol_))
    def exportLiteral(self, outfile, level, name_='TimeDimensionType'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        if self.StartTimes is not None:
            showIndent(outfile, level)
            outfile.write('StartTimes=model_.GeneratedsSuper.gds_parse_datetime("%s"),\n' % self.gds_format_datetime(self.StartTimes, input_name='StartTimes'))
        if self.EndTimes is not None:
            showIndent(outfile, level)
            outfile.write('EndTimes=model_.GeneratedsSuper.gds_parse_datetime("%s"),\n' % self.gds_format_datetime(self.EndTimes, input_name='EndTimes'))
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'StartTimes':
            sval_ = child_.text
            dval_ = self.gds_parse_datetime(sval_)
            self.StartTimes = dval_
            self.StartTimes = self.StartTimes.split()
            self.validate_DateTimeListType(self.StartTimes)    # validate type DateTimeListType
        elif nodeName_ == 'EndTimes':
            sval_ = child_.text
            dval_ = self.gds_parse_datetime(sval_)
            self.EndTimes = dval_
            self.EndTimes = self.EndTimes.split()
            self.validate_DateTimeListType(self.EndTimes)    # validate type DateTimeListType
# end class TimeDimensionType


class DataSetType(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, setkey=None, StationCode=None, StationName=None, InstrumentType=None, ComponentName=None, Location=None, Wavelength=None, D=None, Dmin=None, Dmax=None, Matrix=None, Unit=None, Statistics=None, ResolutionCode=None, DataLevel=None, MeasurementUncertainty=None, Position=None, HeightAGL=None, Values=None):
        self.original_tagname_ = None
        self.setkey = _cast(int, setkey)
        self.StationCode = StationCode
        self.StationName = StationName
        self.InstrumentType = InstrumentType
        self.ComponentName = ComponentName
        self.Location = Location
        self.Wavelength = Wavelength
        self.D = D
        self.Dmin = Dmin
        self.Dmax = Dmax
        self.Matrix = Matrix
        self.Unit = Unit
        self.Statistics = Statistics
        self.ResolutionCode = ResolutionCode
        self.DataLevel = DataLevel
        self.MeasurementUncertainty = MeasurementUncertainty
        self.Position = Position
        self.HeightAGL = HeightAGL
        self.Values = Values
    def factory(*args_, **kwargs_):
        if DataSetType.subclass:
            return DataSetType.subclass(*args_, **kwargs_)
        else:
            return DataSetType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_StationCode(self): return self.StationCode
    def set_StationCode(self, StationCode): self.StationCode = StationCode
    def get_StationName(self): return self.StationName
    def set_StationName(self, StationName): self.StationName = StationName
    def get_InstrumentType(self): return self.InstrumentType
    def set_InstrumentType(self, InstrumentType): self.InstrumentType = InstrumentType
    def get_ComponentName(self): return self.ComponentName
    def set_ComponentName(self, ComponentName): self.ComponentName = ComponentName
    def get_Location(self): return self.Location
    def set_Location(self, Location): self.Location = Location
    def get_Wavelength(self): return self.Wavelength
    def set_Wavelength(self, Wavelength): self.Wavelength = Wavelength
    def get_D(self): return self.D
    def set_D(self, D): self.D = D
    def get_Dmin(self): return self.Dmin
    def set_Dmin(self, Dmin): self.Dmin = Dmin
    def get_Dmax(self): return self.Dmax
    def set_Dmax(self, Dmax): self.Dmax = Dmax
    def get_Matrix(self): return self.Matrix
    def set_Matrix(self, Matrix): self.Matrix = Matrix
    def get_Unit(self): return self.Unit
    def set_Unit(self, Unit): self.Unit = Unit
    def get_Statistics(self): return self.Statistics
    def set_Statistics(self, Statistics): self.Statistics = Statistics
    def get_ResolutionCode(self): return self.ResolutionCode
    def set_ResolutionCode(self, ResolutionCode): self.ResolutionCode = ResolutionCode
    def get_DataLevel(self): return self.DataLevel
    def set_DataLevel(self, DataLevel): self.DataLevel = DataLevel
    def get_MeasurementUncertainty(self): return self.MeasurementUncertainty
    def set_MeasurementUncertainty(self, MeasurementUncertainty): self.MeasurementUncertainty = MeasurementUncertainty
    def get_Position(self): return self.Position
    def set_Position(self, Position): self.Position = Position
    def get_HeightAGL(self): return self.HeightAGL
    def set_HeightAGL(self, HeightAGL): self.HeightAGL = HeightAGL
    def get_Values(self): return self.Values
    def set_Values(self, Values): self.Values = Values
    def get_setkey(self): return self.setkey
    def set_setkey(self, setkey): self.setkey = setkey
    def validate_LocationType(self, value):
        # Validate type LocationType, a restriction on xs:string.
        pass
    def validate_ValueListType(self, value):
        # Validate type ValueListType, a restriction on xs:float.
        pass
    def hasContent_(self):
        if (
            self.StationCode is not None or
            self.StationName is not None or
            self.InstrumentType is not None or
            self.ComponentName is not None or
            self.Location is not None or
            self.Wavelength is not None or
            self.D is not None or
            self.Dmin is not None or
            self.Dmax is not None or
            self.Matrix is not None or
            self.Unit is not None or
            self.Statistics is not None or
            self.ResolutionCode is not None or
            self.DataLevel is not None or
            self.MeasurementUncertainty is not None or
            self.Position is not None or
            self.HeightAGL is not None or
            self.Values is not None
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='DataSetType', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='DataSetType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='DataSetType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='DataSetType'):
        if self.setkey is not None and 'setkey' not in already_processed:
            already_processed.add('setkey')
            outfile.write(' setkey="%s"' % self.gds_format_integer(self.setkey, input_name='setkey'))
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='DataSetType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.StationCode is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sStationCode>%s</%sStationCode>%s' % (namespace_, self.gds_format_string(quote_xml(self.StationCode).encode(ExternalEncoding), input_name='StationCode'), namespace_, eol_))
        if self.StationName is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sStationName>%s</%sStationName>%s' % (namespace_, self.gds_format_string(quote_xml(self.StationName).encode(ExternalEncoding), input_name='StationName'), namespace_, eol_))
        if self.InstrumentType is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sInstrumentType>%s</%sInstrumentType>%s' % (namespace_, self.gds_format_string(quote_xml(self.InstrumentType).encode(ExternalEncoding), input_name='InstrumentType'), namespace_, eol_))
        if self.ComponentName is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sComponentName>%s</%sComponentName>%s' % (namespace_, self.gds_format_string(quote_xml(self.ComponentName).encode(ExternalEncoding), input_name='ComponentName'), namespace_, eol_))
        if self.Location is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sLocation>%s</%sLocation>%s' % (namespace_, self.gds_format_string(quote_xml(self.Location).encode(ExternalEncoding), input_name='Location'), namespace_, eol_))
        if self.Wavelength is not None:
            self.Wavelength.export(outfile, level, namespace_, name_='Wavelength', pretty_print=pretty_print)
        if self.D is not None:
            self.D.export(outfile, level, namespace_, name_='D', pretty_print=pretty_print)
        if self.Dmin is not None:
            self.Dmin.export(outfile, level, namespace_, name_='Dmin', pretty_print=pretty_print)
        if self.Dmax is not None:
            self.Dmax.export(outfile, level, namespace_, name_='Dmax', pretty_print=pretty_print)
        if self.Matrix is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sMatrix>%s</%sMatrix>%s' % (namespace_, self.gds_format_string(quote_xml(self.Matrix).encode(ExternalEncoding), input_name='Matrix'), namespace_, eol_))
        if self.Unit is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sUnit>%s</%sUnit>%s' % (namespace_, self.gds_format_string(quote_xml(self.Unit).encode(ExternalEncoding), input_name='Unit'), namespace_, eol_))
        if self.Statistics is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sStatistics>%s</%sStatistics>%s' % (namespace_, self.gds_format_string(quote_xml(self.Statistics).encode(ExternalEncoding), input_name='Statistics'), namespace_, eol_))
        if self.ResolutionCode is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sResolutionCode>%s</%sResolutionCode>%s' % (namespace_, self.gds_format_string(quote_xml(self.ResolutionCode).encode(ExternalEncoding), input_name='ResolutionCode'), namespace_, eol_))
        if self.DataLevel is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sDataLevel>%s</%sDataLevel>%s' % (namespace_, self.gds_format_string(quote_xml(self.DataLevel).encode(ExternalEncoding), input_name='DataLevel'), namespace_, eol_))
        if self.MeasurementUncertainty is not None:
            self.MeasurementUncertainty.export(outfile, level, namespace_, name_='MeasurementUncertainty', pretty_print=pretty_print)
        if self.Position is not None:
            self.Position.export(outfile, level, namespace_, name_='Position', pretty_print=pretty_print)
        if self.HeightAGL is not None:
            self.HeightAGL.export(outfile, level, namespace_, name_='HeightAGL', pretty_print=pretty_print)
        if self.Values is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sValues>%s</%sValues>%s' % (namespace_, self.gds_format_float_list(self.Values, input_name='Values'), namespace_, eol_))
    def exportLiteral(self, outfile, level, name_='DataSetType'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        if self.setkey is not None and 'setkey' not in already_processed:
            already_processed.add('setkey')
            showIndent(outfile, level)
            outfile.write('setkey=%d,\n' % (self.setkey,))
    def exportLiteralChildren(self, outfile, level, name_):
        if self.StationCode is not None:
            showIndent(outfile, level)
            outfile.write('StationCode=%s,\n' % quote_python(self.StationCode).encode(ExternalEncoding))
        if self.StationName is not None:
            showIndent(outfile, level)
            outfile.write('StationName=%s,\n' % quote_python(self.StationName).encode(ExternalEncoding))
        if self.InstrumentType is not None:
            showIndent(outfile, level)
            outfile.write('InstrumentType=%s,\n' % quote_python(self.InstrumentType).encode(ExternalEncoding))
        if self.ComponentName is not None:
            showIndent(outfile, level)
            outfile.write('ComponentName=%s,\n' % quote_python(self.ComponentName).encode(ExternalEncoding))
        if self.Location is not None:
            showIndent(outfile, level)
            outfile.write('Location=%s,\n' % quote_python(self.Location).encode(ExternalEncoding))
        if self.Wavelength is not None:
            showIndent(outfile, level)
            outfile.write('Wavelength=model_.WavelengthType(\n')
            self.Wavelength.exportLiteral(outfile, level, name_='Wavelength')
            showIndent(outfile, level)
            outfile.write('),\n')
        if self.D is not None:
            showIndent(outfile, level)
            outfile.write('D=model_.ParticleDiameterType(\n')
            self.D.exportLiteral(outfile, level, name_='D')
            showIndent(outfile, level)
            outfile.write('),\n')
        if self.Dmin is not None:
            showIndent(outfile, level)
            outfile.write('Dmin=model_.ParticleDiameterType(\n')
            self.Dmin.exportLiteral(outfile, level, name_='Dmin')
            showIndent(outfile, level)
            outfile.write('),\n')
        if self.Dmax is not None:
            showIndent(outfile, level)
            outfile.write('Dmax=model_.ParticleDiameterType(\n')
            self.Dmax.exportLiteral(outfile, level, name_='Dmax')
            showIndent(outfile, level)
            outfile.write('),\n')
        if self.Matrix is not None:
            showIndent(outfile, level)
            outfile.write('Matrix=%s,\n' % quote_python(self.Matrix).encode(ExternalEncoding))
        if self.Unit is not None:
            showIndent(outfile, level)
            outfile.write('Unit=%s,\n' % quote_python(self.Unit).encode(ExternalEncoding))
        if self.Statistics is not None:
            showIndent(outfile, level)
            outfile.write('Statistics=%s,\n' % quote_python(self.Statistics).encode(ExternalEncoding))
        if self.ResolutionCode is not None:
            showIndent(outfile, level)
            outfile.write('ResolutionCode=%s,\n' % quote_python(self.ResolutionCode).encode(ExternalEncoding))
        if self.DataLevel is not None:
            showIndent(outfile, level)
            outfile.write('DataLevel=%s,\n' % quote_python(self.DataLevel).encode(ExternalEncoding))
        if self.MeasurementUncertainty is not None:
            showIndent(outfile, level)
            outfile.write('MeasurementUncertainty=model_.MeasurementUncertainty(\n')
            self.MeasurementUncertainty.exportLiteral(outfile, level)
            showIndent(outfile, level)
            outfile.write('),\n')
        if self.Position is not None:
            showIndent(outfile, level)
            outfile.write('Position=model_.Position(\n')
            self.Position.exportLiteral(outfile, level)
            showIndent(outfile, level)
            outfile.write('),\n')
        if self.HeightAGL is not None:
            showIndent(outfile, level)
            outfile.write('HeightAGL=model_.HeightAGL(\n')
            self.HeightAGL.exportLiteral(outfile, level)
            showIndent(outfile, level)
            outfile.write('),\n')
        if self.Values is not None:
            showIndent(outfile, level)
            outfile.write('Values=%f,\n' % self.Values)
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('setkey', node)
        if value is not None and 'setkey' not in already_processed:
            already_processed.add('setkey')
            try:
                self.setkey = int(value)
            except ValueError as exp:
                raise_parse_error(node, 'Bad integer attribute: %s' % exp)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'StationCode':
            StationCode_ = child_.text
            StationCode_ = self.gds_validate_string(StationCode_, node, 'StationCode')
            self.StationCode = StationCode_
        elif nodeName_ == 'StationName':
            StationName_ = child_.text
            StationName_ = self.gds_validate_string(StationName_, node, 'StationName')
            self.StationName = StationName_
        elif nodeName_ == 'InstrumentType':
            InstrumentType_ = child_.text
            InstrumentType_ = self.gds_validate_string(InstrumentType_, node, 'InstrumentType')
            self.InstrumentType = InstrumentType_
        elif nodeName_ == 'ComponentName':
            ComponentName_ = child_.text
            ComponentName_ = self.gds_validate_string(ComponentName_, node, 'ComponentName')
            self.ComponentName = ComponentName_
        elif nodeName_ == 'Location':
            Location_ = child_.text
            Location_ = self.gds_validate_string(Location_, node, 'Location')
            self.Location = Location_
            self.validate_LocationType(self.Location)    # validate type LocationType
        elif nodeName_ == 'Wavelength':
            obj_ = WavelengthType.factory()
            obj_.build(child_)
            self.Wavelength = obj_
            obj_.original_tagname_ = 'Wavelength'
        elif nodeName_ == 'D':
            obj_ = ParticleDiameterType.factory()
            obj_.build(child_)
            self.D = obj_
            obj_.original_tagname_ = 'D'
        elif nodeName_ == 'Dmin':
            obj_ = ParticleDiameterType.factory()
            obj_.build(child_)
            self.Dmin = obj_
            obj_.original_tagname_ = 'Dmin'
        elif nodeName_ == 'Dmax':
            obj_ = ParticleDiameterType.factory()
            obj_.build(child_)
            self.Dmax = obj_
            obj_.original_tagname_ = 'Dmax'
        elif nodeName_ == 'Matrix':
            Matrix_ = child_.text
            Matrix_ = self.gds_validate_string(Matrix_, node, 'Matrix')
            self.Matrix = Matrix_
        elif nodeName_ == 'Unit':
            Unit_ = child_.text
            Unit_ = self.gds_validate_string(Unit_, node, 'Unit')
            self.Unit = Unit_
        elif nodeName_ == 'Statistics':
            Statistics_ = child_.text
            Statistics_ = self.gds_validate_string(Statistics_, node, 'Statistics')
            self.Statistics = Statistics_
        elif nodeName_ == 'ResolutionCode':
            ResolutionCode_ = child_.text
            ResolutionCode_ = self.gds_validate_string(ResolutionCode_, node, 'ResolutionCode')
            self.ResolutionCode = ResolutionCode_
        elif nodeName_ == 'DataLevel':
            DataLevel_ = child_.text
            DataLevel_ = self.gds_validate_string(DataLevel_, node, 'DataLevel')
            self.DataLevel = DataLevel_
        elif nodeName_ == 'MeasurementUncertainty':
            obj_ = MeasurementUncertainty.factory()
            obj_.build(child_)
            self.MeasurementUncertainty = obj_
            obj_.original_tagname_ = 'MeasurementUncertainty'
        elif nodeName_ == 'Position':
            obj_ = Position.factory()
            obj_.build(child_)
            self.Position = obj_
            obj_.original_tagname_ = 'Position'
        elif nodeName_ == 'HeightAGL':
            obj_ = HeightAGL.factory()
            obj_.build(child_)
            self.HeightAGL = obj_
            obj_.original_tagname_ = 'HeightAGL'
        elif nodeName_ == 'Values':
            Values_ = child_.text
            Values_ = self.gds_validate_float_list(Values_, node, 'Values')
            self.Values = Values_
            self.Values = self.Values.split()
            self.validate_ValueListType(self.Values)    # validate type ValueListType
# end class DataSetType


class Statistics(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self):
        self.original_tagname_ = None
    def factory(*args_, **kwargs_):
        if Statistics.subclass:
            return Statistics.subclass(*args_, **kwargs_)
        else:
            return Statistics(*args_, **kwargs_)
    factory = staticmethod(factory)
    def hasContent_(self):
        if (

        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='Statistics', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Statistics')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='Statistics', pretty_print=pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='Statistics'):
        pass
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='Statistics', fromsubclass_=False, pretty_print=True):
        pass
    def exportLiteral(self, outfile, level, name_='Statistics'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        pass
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class Statistics


class MeasurementUncertainty(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, unit=None, valueOf_=None):
        self.original_tagname_ = None
        self.unit = _cast(None, unit)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if MeasurementUncertainty.subclass:
            return MeasurementUncertainty.subclass(*args_, **kwargs_)
        else:
            return MeasurementUncertainty(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_unit(self): return self.unit
    def set_unit(self, unit): self.unit = unit
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='MeasurementUncertainty', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MeasurementUncertainty')
        if self.hasContent_():
            outfile.write('>')
            outfile.write(str(self.valueOf_).encode(ExternalEncoding))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='MeasurementUncertainty', pretty_print=pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='MeasurementUncertainty'):
        if self.unit is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            outfile.write(' unit=%s' % (self.gds_format_string(quote_attrib(self.unit).encode(ExternalEncoding), input_name='unit'), ))
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='MeasurementUncertainty', fromsubclass_=False, pretty_print=True):
        pass
    def exportLiteral(self, outfile, level, name_='MeasurementUncertainty'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
        showIndent(outfile, level)
        outfile.write('valueOf_ = """%s""",\n' % (self.valueOf_,))
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        if self.unit is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            showIndent(outfile, level)
            outfile.write('unit="%s",\n' % (self.unit,))
    def exportLiteralChildren(self, outfile, level, name_):
        pass
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('unit', node)
        if value is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            self.unit = value
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class MeasurementUncertainty


class Position(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, Point=None):
        self.original_tagname_ = None
        self.Point = Point
    def factory(*args_, **kwargs_):
        if Position.subclass:
            return Position.subclass(*args_, **kwargs_)
        else:
            return Position(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Point(self): return self.Point
    def set_Point(self, Point): self.Point = Point
    def hasContent_(self):
        if (
            self.Point is not None
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='Position', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS" xmlns:gml="http://www.opengis.net/gml/3.2" ', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Position')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='Position', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='Position'):
        pass
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='Position', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Point is not None:
            showIndent(outfile, level, pretty_print)
            outfile.write('<%sPoint>%s</%sPoint>%s' % ('gml:', self.gds_format_string(quote_xml(self.Point).encode(ExternalEncoding), input_name='Point'), 'gml:', eol_))
    def exportLiteral(self, outfile, level, name_='Position'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        if self.Point is not None:
            showIndent(outfile, level)
            outfile.write('Point=%s,\n' % quote_python(self.Point).encode(ExternalEncoding))
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Point':
            Point_ = child_.text
            Point_ = self.gds_validate_string(Point_, node, 'Point')
            self.Point = Point_
# end class Position


class HeightAGL(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, unit=None, valueOf_=None):
        self.original_tagname_ = None
        self.unit = _cast(None, unit)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if HeightAGL.subclass:
            return HeightAGL.subclass(*args_, **kwargs_)
        else:
            return HeightAGL(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_unit(self): return self.unit
    def set_unit(self, unit): self.unit = unit
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='HeightAGL', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='HeightAGL')
        if self.hasContent_():
            outfile.write('>')
            outfile.write(str(self.valueOf_).encode(ExternalEncoding))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='HeightAGL', pretty_print=pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='HeightAGL'):
        if self.unit is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            outfile.write(' unit=%s' % (self.gds_format_string(quote_attrib(self.unit).encode(ExternalEncoding), input_name='unit'), ))
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='HeightAGL', fromsubclass_=False, pretty_print=True):
        pass
    def exportLiteral(self, outfile, level, name_='HeightAGL'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
        showIndent(outfile, level)
        outfile.write('valueOf_ = """%s""",\n' % (self.valueOf_,))
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        if self.unit is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            showIndent(outfile, level)
            outfile.write('unit="%s",\n' % (self.unit,))
    def exportLiteralChildren(self, outfile, level, name_):
        pass
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('unit', node)
        if value is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            self.unit = value
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class HeightAGL


class WavelengthType(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, unit=None, valueOf_=None):
        self.original_tagname_ = None
        self.unit = _cast(None, unit)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if WavelengthType.subclass:
            return WavelengthType.subclass(*args_, **kwargs_)
        else:
            return WavelengthType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_unit(self): return self.unit
    def set_unit(self, unit): self.unit = unit
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='WavelengthType', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='WavelengthType')
        if self.hasContent_():
            outfile.write('>')
            outfile.write(str(self.valueOf_).encode(ExternalEncoding))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='WavelengthType', pretty_print=pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='WavelengthType'):
        if self.unit is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            outfile.write(' unit=%s' % (self.gds_format_string(quote_attrib(self.unit).encode(ExternalEncoding), input_name='unit'), ))
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='WavelengthType', fromsubclass_=False, pretty_print=True):
        pass
    def exportLiteral(self, outfile, level, name_='WavelengthType'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
        showIndent(outfile, level)
        outfile.write('valueOf_ = """%s""",\n' % (self.valueOf_,))
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        if self.unit is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            showIndent(outfile, level)
            outfile.write('unit="%s",\n' % (self.unit,))
    def exportLiteralChildren(self, outfile, level, name_):
        pass
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('unit', node)
        if value is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            self.unit = value
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class WavelengthType


class ParticleDiameterType(GeneratedsSuper):
    subclass = None
    superclass = None
    def __init__(self, unit=None, valueOf_=None):
        self.original_tagname_ = None
        self.unit = _cast(None, unit)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if ParticleDiameterType.subclass:
            return ParticleDiameterType.subclass(*args_, **kwargs_)
        else:
            return ParticleDiameterType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_unit(self): return self.unit
    def set_unit(self, unit): self.unit = unit
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='ebas:', name_='ParticleDiameterType', namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ParticleDiameterType')
        if self.hasContent_():
            outfile.write('>')
            outfile.write(str(self.valueOf_).encode(ExternalEncoding))
            self.exportChildren(outfile, level + 1, namespace_='ebas:', name_='ParticleDiameterType', pretty_print=pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='ebas:', name_='ParticleDiameterType'):
        if self.unit is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            outfile.write(' unit=%s' % (self.gds_format_string(quote_attrib(self.unit).encode(ExternalEncoding), input_name='unit'), ))
    def exportChildren(self, outfile, level, namespace_='ebas:', name_='ParticleDiameterType', fromsubclass_=False, pretty_print=True):
        pass
    def exportLiteral(self, outfile, level, name_='ParticleDiameterType'):
        level += 1
        already_processed = set()
        self.exportLiteralAttributes(outfile, level, already_processed, name_)
        if self.hasContent_():
            self.exportLiteralChildren(outfile, level, name_)
        showIndent(outfile, level)
        outfile.write('valueOf_ = """%s""",\n' % (self.valueOf_,))
    def exportLiteralAttributes(self, outfile, level, already_processed, name_):
        if self.unit is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            showIndent(outfile, level)
            outfile.write('unit="%s",\n' % (self.unit,))
    def exportLiteralChildren(self, outfile, level, name_):
        pass
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('unit', node)
        if value is not None and 'unit' not in already_processed:
            already_processed.add('unit')
            self.unit = value
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class ParticleDiameterType


GDSClassesMapping = {
    'D': ParticleDiameterType,
    'ResultSet': ResultSetType,
    'DataSet': DataSetType,
    'Dmin': ParticleDiameterType,
    'TimeDimension': TimeDimensionType,
    'Wavelength': WavelengthType,
    'Dmax': ParticleDiameterType,
}


USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""


def usage():
    print(USAGE_TEXT)
    sys.exit(1)


def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass


def parse(inFileName, silence=False):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Result'
        rootClass = Result
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    if not silence:
        sys.stdout.write('<?xml version="1.0" ?>\n')
        rootObj.export(
            sys.stdout, 0, name_=rootTag,
            namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"',
            pretty_print=True)
    return rootObj


def parseEtree(inFileName, silence=False):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Result'
        rootClass = Result
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    mapping = {}
    rootElement = rootObj.to_etree(None, name_=rootTag, mapping_=mapping)
    reverse_mapping = rootObj.gds_reverse_node_mapping(mapping)
    if not silence:
        content = etree_.tostring(
            rootElement, pretty_print=True,
            xml_declaration=True, encoding="utf-8")
        sys.stdout.write(content)
        sys.stdout.write('\n')
    return rootObj, rootElement, mapping, reverse_mapping


def parseString(inString, silence=False):
    from six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Result'
        rootClass = Result
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    if not silence:
        sys.stdout.write('<?xml version="1.0" ?>\n')
        rootObj.export(
            sys.stdout, 0, name_=rootTag,
            namespacedef_='xmlns:ebas="http://ebas.nilu.no/EBAS"')
    return rootObj


def parseLiteral(inFileName, silence=False):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Result'
        rootClass = Result
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    if not silence:
        sys.stdout.write('#from ebas_generated import *\n\n')
        sys.stdout.write('import ebas_generated as model_\n\n')
        sys.stdout.write('rootObj = model_.rootClass(\n')
        rootObj.exportLiteral(sys.stdout, 0, name_=rootTag)
        sys.stdout.write(')\n')
    return rootObj


def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()


__all__ = [
    "DataSetType",
    "HeightAGL",
    "MeasurementUncertainty",
    "ParticleDiameterType",
    "Position",
    "Result",
    "ResultSetType",
    "Statistics",
    "TimeDimensionType",
    "WavelengthType"
]



"""
ebas/domain/masterdata/pr.py
$Id: pr.py 2088 2018-06-29 13:46:38Z pe $

EBAS Masterdata Class for Projects

This module implements the class EbasMasterPR.

History:
V.1.0.0  2014-02-25  pe  initial version

"""

from .. import EbasDomainError
from .offline_masterdata import PROfflineMasterData
from .base import EbasMasterBase

class EbasMasterPR(EbasMasterBase, PROfflineMasterData):
    """
    Domain Class for station masterdata.
    Objects of this class do not represent entities, this class provides class
    methods for handling stations and checking them against master data.
    Stations master data are retrieved from database or from offline storage
    when no database access is possible.
    """

    # read offline masterdata for PR (projects)
    # Those are fallback values, will be read from database as soon as possible.
    PROfflineMasterData.read_pickle_file()

    def __init__(self, dbh=None):
        """
        Read pr masterdata if dbh is provided.
        """
        PROfflineMasterData.__init__(self)
        if not self.__class__.INIT and dbh:
            self.from_db(dbh)

    @classmethod
    def lookup_project(cls, project_acronym):
        """
        Lookup a project acronym (from command line parameter).
        Valid are:
            - full matching project acronym
        Parameters:
            project_acronym    string with project acronym
        Returns:
            string with project acronym:
        Raises:
            EbasDomainError if project acronym is invalid
        """
        if project_acronym in cls.META:
            # string can be found as a project acronym
            return project_acronym
        else:
            raise EbasDomainError("non existing project acronym '{}'"
                                  .format(project_acronym))



# coding: utf-8
"""
$Id$

output functionality for EBAS NetCDF
"""

import os.path
import sys
import itertools
from collections import defaultdict
from six import string_types
from .base import EbasNetcdfBase
from ..base import FLAGS_ALL, EBAS_IOFORMAT_NETCDF1, \
    SUPPRESS_SORT_VARIABLES, SUPPRESS_METADATA_OCCURRENCE

try:
    import netCDF4
    import numpy
    NETCDFSTRING = numpy.unicode_ if numpy.__version__ < '2.0' else numpy.str_
except ImportError:
    # pydap and numpy are NOT dependencies for ebas-io. Thus, ebas-io can
    # be used w/o this modules.
    # However the functionality is limited. An error message will be thrown
    # in the entry points of ebas opendap writer (methods write, make_dataset)
    pass


from ..basefile.write import VAL, FLG, MTA

class EbasNetCDFPartialWrite(EbasNetcdfBase):  # pylint: disable=R0901, W0223,
    # R0901: Too many ancestors
    # This is a quite big class, therefore we ignore this warning
    # W0223: ... is abstract in base class but is not overridden
    # this class is abstract as well (does not have any NotImplementedError)
    """
    Output of I/O object to NetCDF
    """

    def __init__(self, *args, **kwargs):
        """
        Initialize the object.
        """
        super(EbasNetCDFPartialWrite, self).__init__(*args, **kwargs)
        # dimensions added to the netCFD file only used during write():
        self.dims = None

    def write(self, createfiles=False, destdir=None, xmlwrap=False,
              fileobj=None, flags=FLAGS_ALL,  # @UnusedVariable
              datadef='EBAS_1.1', metadata_options=0, suppress=0):
        # pylint: disable=R0913,W0613
        # R0913: Too many arguments
        # W0613: Unused argument flags (not used for NetCDF)
        """
        Writes the I/O object to a NetCDF file (or stdout)
        Parameters:
            createfiles   write to file? else write to stdout
                          (special case for fileobj)
            destdir       destination directory path for files
            xmlwrap       wrap output in xml container
            fileobj       write to filehandle
            flags         flag column options: FLAGS_ONE_OR_ALL (default),
                          FLAGS_COMPRESS, FLAGS_ALL, FLAGS_NONE
            datadef       ebas data definition (EBAS_1 or EBAS_1.1)
            metadata_options
                          options for output, bitfield:
                          currently only EBAS_IOMETADATA_OPTION_SETKEY
            suppress      suppress selected consolidations (bitfield):
                              SUPPRESS_SORT_VARIABLES
                              SUPPRESS_METADATA_OCCURRENCE
        Returns:
            None
        """
        self.check_requirements()

        # prepare write used without opening the output file because the netCDF4
        # module cannot handle output to open files. This is somewhere deep in
        # the NetCDF C library and cannot be changed easily. Instead we will
        # pass the file name to netCDF4.
        # Instead, the file logic needs to be handled here, includeing some
        # workaround for stdout/stream outpot (diskleess object, seralized in
        # the end)
        self.prepare_write(
            False, createfiles=createfiles, destdir=destdir, xmlwrap=xmlwrap,
            fileobj=fileobj, datadef=datadef, metadata_options=metadata_options,
            suppress=suppress)

        if createfiles:
            if destdir:
                fnam = os.path.join(destdir, self.metadata.filename)
            else:
                fnam = self.metadata.filename
            # NETCDF3_64BIT_OFFSET ???
            ncfile = netCDF4.Dataset(fnam, mode='w',
                                     format='NETCDF4')
            self.logger.info("writing to file " + fnam)

        else:
            ncfile = netCDF4.Dataset(self.metadata.filename, diskless=True,
                                     mode='w', format='NETCDF4')
        self.ncfile = ncfile

        self.add_global_attributes(metadata_options)
        self.add_time_variables()
        self.add_variables()
        #self.parse_characteristics(ncfile)
        #self.create_variables(ncfile, var_time)
        if not createfiles:
            sys.stdout.write(str(ncfile))
        self.finish_write(createfiles, xmlwrap)
        ncfile.close()
        #if xmlwrap:
        #    xmlwrap_filetrailer()
        #self.set_data(flags=flags)
        self.write_indexdb()

    def check_requirements(self):
        """
        Check requirements for opendap output.
        pydap and numpy are NOT dependencies for ebas-io generally.
        Thus, ebas-io can  be used w/o this modules.
        However the functionality is limited. An error message is thrown here
        and no output produced.
        # Info: make dataset is also a direct entry point (from the webservice)
        """
        # netCDF4 and numpy are NOT dependencies for ebas-io. Thus, ebas-io can
        # be used w/o this modules.
        # However the functionality is limited. An error message is thrown
        # here and no output produced.
        try:
            import netCDF4  # @UnusedImport pylint: disable=W0612, W0621
            # W0612: Unused variable 'netCDF4'
            # W0621: Redefining name 'netCDF4' from outer scope
            # Unused import: netCDF4
            # --> this line is just used to get a nice error message instead
            # of an exception in case of requirements not been installed
        except ImportError:
            self.error("The python netCDF4 module is not installed. NetCDF "
                       "output is not available.")
            return
        try:
            import numpy  # @UnusedImport pylint: disable=W0612, W0621
            # W0612: Unused variable 'netCDF4'
            # W0621: Redefining name 'netCDF4' from outer scope
            # Unused import: netCDF4
            # --> this line is just used to get a nice error message instead
            # of an exception in case of requirements not been installed
        except ImportError:
            self.error("The python numpy module is not installed. NetCDF "
                       "output is not available.")
            return

    def add_global_attributes(self, metadata_options):
        """
        Add global attributes.
        See generate_global_acdd_attributes and generate_global_ebas_attributes
        for details.
        """
        # ACDD atttributes
        for att in self.generate_global_acdd_attributes(metadata_options):
            self.ncfile.setncattr(att[0], att[1])
        for att in self.generate_global_ebas_attributes(
                version=1 if self.__class__.IOFORMAT == EBAS_IOFORMAT_NETCDF1
                else 2):
            self.ncfile.setncattr(att[0], att[1])

    def add_time_variables(self):
        """
        Add the time variables and the time dimenstion.
        """
        self.ncfile.createDimension('time', len(self.sample_times))
        self.ncfile.createDimension('metadata_time',
                                    len(self.metadata_intervals))
        self.ncfile.createDimension('tbnds', 2)

        # Create time variable
        var_time = self.ncfile.createVariable('time', numpy.float64, ('time', ))
        # Add varaible attributes
        var_time.standard_name = 'time'
        var_time.long_name = 'time of measurement'
        var_time.units = 'days since 1900-01-01 00:00:00 UTC'
        var_time.axis = 'T'
        var_time.calendar = 'gregorian'
        var_time.bounds = "time_bnds"
        # values for the time variable:
        var_time[:] = netCDF4.date2num(
            [intv[0]+(intv[1]-intv[0])/2 for intv in self.sample_times],
            var_time.units, calendar=var_time.calendar)

        # Create time_bounds variable
        var_time_bounds = self.ncfile.createVariable(
            'time_bnds', numpy.float64, ('time', 'tbnds',))
        
        # Add variable attributes
        var_time_bounds.long_name = 'time bounds for measurement'
        # CF-1.8 (Chapter 7.1): for cell boundaries those attributes are NOT
        # reccommended: units, standard_name, axis, positive, calendar,
        #   leap_month, leap_year
        # var_time_bounds.standard_name = 'time'
        # var_time_bounds.units = 'days since 1900-01-01 00:00:00 UTC'

        # values for the var_time_bounds variable:
        var_time_bounds[:] = numpy.column_stack((
            netCDF4.date2num([intv[0] for intv in self.sample_times],
                             var_time.units, calendar=var_time.calendar),
            netCDF4.date2num([intv[1] for intv in self.sample_times],
                             var_time.units, calendar=var_time.calendar)))

        # Create metadata_time variable
        var_metadata_time = self.ncfile.createVariable(
            'metadata_time', numpy.float64, ('metadata_time', ))
        # Add varaible attributes
        var_metadata_time.standard_name = 'time'
        var_metadata_time.long_name = 'time of ebas metadata intervals'
        var_metadata_time.units = 'days since 1900-01-01 00:00:00 UTC'
        var_metadata_time.axis = 'T'
        var_metadata_time.calendar = 'gregorian'
        var_metadata_time.bounds = "metadata_time_bnds"
        # values for the metadata_time variable:
        var_metadata_time[:] = netCDF4.date2num(
            [intv[0]+(intv[1]-intv[0])/2 for intv in self.metadata_intervals],
            var_metadata_time.units, calendar=var_metadata_time.calendar)

        # Create time_bounds variable
        var_metadata_time_bounds = self.ncfile.createVariable(
            'metadata_time_bnds', numpy.float64, ('metadata_time', 'tbnds',))
        # Add variable attributes
        var_metadata_time_bounds.long_name = \
            'time bounds for ebas metadata intervals'
        # CF-1.8 (Chapter 7.1): for cell boundaries those attributes are NOT
        # reccommended: units, standard_name, axis, positive, calendar,
        #   leap_month, leap_year
        # var_metadata_time_bounds.standard_name = 'time'
        # var_metadata_time_bounds.units = 'days since 1900-01-01 00:00:00 UTC'
        # var_metadata_time_bounds.calendar = 'gregorian'

        # values for the metadata_time_bounds variable:
        var_metadata_time_bounds[:] = numpy.column_stack((
            netCDF4.date2num([intv[0] for intv in self.metadata_intervals],
                             var_metadata_time.units,
                             calendar=var_metadata_time.calendar),
            netCDF4.date2num([intv[1] for intv in self.metadata_intervals],
                             var_metadata_time.units,
                             calendar=var_metadata_time.calendar)))

    def add_var_dimensions(self, cdm_var):
        """
        Add additional dimensions needed by the cdm variable.
        Parameters:
            cdm_var    the cdm variable description (dict)
        Returns:
            None
        """
        for i in range(len(cdm_var['dimensions'])):
            dim = cdm_var['dimensions'][i]
            if dim not in self.dims:
                self.dims.add(dim)
                self.ncfile.createDimension(
                    dim, len(cdm_var['dimextents'][i]))
                if isinstance(cdm_var['dimextents'][i][0], string_types):
                    var = self.ncfile.createVariable(dim, NETCDFSTRING,
                                                     (dim, ))
                    var[:] = numpy.array([ext
                                          for ext in cdm_var['dimextents'][i]],
                                         NETCDFSTRING)
                elif isinstance(cdm_var['dimextents'][i][0], int):
                    var = self.ncfile.createVariable(dim, numpy.int32, (dim, ))
                    var[:] = numpy.array([ext
                                          for ext in cdm_var['dimextents'][i]],
                                         numpy.int32)
                else:
                    var = self.ncfile.createVariable(dim, numpy.float64,
                                                     (dim, ))
                    var[:] = numpy.array([ext
                                          for ext in cdm_var['dimextents'][i]],
                                         numpy.float64)

    def add_var_attributes(self, var, cdm_var):
        """
        Set the variable attributes for a cdm variable.
        Parameters:
            var        the netCDF variable
            cdm_var    the cdm variable description (dict)
        Returns:
            None
        """
        # use the first variable (of all dimensions) for the attributes
        ebas_var_ind = cdm_var['ebas_variables'][0]
        cfname, cfunit = self.cfparams(
            self.get_meta_for_var(ebas_var_ind, 'regime'),
            self.get_meta_for_var(ebas_var_ind, 'matrix'),
            self.get_meta_for_var(ebas_var_ind, 'comp_name'),
            self.get_meta_for_var(ebas_var_ind, 'statistics'),
            self.get_meta_for_var(ebas_var_ind, 'unit'),
            self.get_meta_for_var(ebas_var_ind, 'datalevel'))
        if cdm_var['var_type'] == FLG:
            var.setncatts({'standard_name': "status_flag"})
            var.setncatts({'missing_value': numpy.int32(0)})
            var.setncatts({'_FillValue': numpy.int32(0)})
            var.setncatts({'units': '1'})
        elif cdm_var['var_type'] == MTA:
            var.setncatts({
                'long_name':
                    "ebas metadata for different time intervals; json encoded"})
        else:
            if cfname:
                var.setncatts({'standard_name': cfname})
            var.setncatts({'missing_value': numpy.float64(numpy.nan)})
            var.setncatts({'_FillValue': numpy.float64(numpy.nan)})
            if cfunit:
                var.setncatts({'units': cfunit})
            else:
                var.setncatts({'units':
                               self.get_meta_for_var(ebas_var_ind, 'unit')})
            var.setncatts({
                'ancillary_variables':
                    "{0}_qc {0}_ebasmetadata".format(cdm_var['cdm_varname'])})
            var.setncatts({'cell_methods': self.cell_methods(ebas_var_ind)})
            if self.__class__.IOFORMAT != EBAS_IOFORMAT_NETCDF1:
                # In version 2 we use all the ebas_XXX attributes:
                atts = defaultdict(int)
                for var_index in cdm_var['ebas_variables']:
                    # In case of addtitional dimensions (characteristics), we
                    # need to make sure, only attributes which are the same for
                    # all variables are used.
                    for att in self.generate_variable_ebas_attributes(
                            var_index=var_index):
                        atts[(att[0], att[1])] += 1
                for tag_val in atts:
                    if atts[tag_val] == len(cdm_var['ebas_variables']):
                        var.setncatts({tag_val[0]: tag_val[1]})

    def add_var_data(self, var, cdm_var):
        """
        Set the variable data for a cdm variable.
        Parameters:
            var        the netCDF variable
            cdm_var    the cdm variable description (dict)
        Returns:
            None
        """
        maxlen_ebasmeta = 3   # TODO?
        # now fill the data according to the dims list
        dims = [[tuple([cdm_var['dimensions'][i], k]) for k in cdm_var['dimextents'][i]] for i in range(len(cdm_var['dimensions']))]
        indxs = [[slice(j, j+1) for j in range(len(cdm_var['dimextents'][i]))] for i in range(len(cdm_var['dimensions']))]
        index_perm = itertools.product(*indxs) # index permutations
        for elem in itertools.product(*dims):
            data_ind = next(index_perm)
            idx = cdm_var['variable_dimensions'].index(elem)
            var_index = cdm_var['ebas_variables'][idx]
            if cdm_var['var_type'] == VAL:
                data = numpy.array(
                    self.variables[var_index].values_,
                    dtype=float)
            if cdm_var['var_type'] == FLG:
                # prepare variable flags according to dimensions
                fdim = cdm_var['flag_dimsize']
                tdim = len(self.sample_times)
                data = numpy.empty(shape=(fdim, tdim), dtype=numpy.int32)
                for i in range(fdim):
                    lst = [flag[i] if len(flag) >= i+1 else 0 for flag in self.variables[var_index].flags]
                    data[i] = numpy.array(lst, dtype=numpy.int32)
            if cdm_var['var_type'] == MTA:
                # prepare variable metadata according to metadata_time dimension
                #dtype = 'U{}'.format(maxlen_ebasmeta)
                ebasmeta = [
                    mdi[2].ebas_metadata_json(
                        var_index=var_index,
                        version=1 if self.__class__.IOFORMAT == \
                            EBAS_IOFORMAT_NETCDF1 else 2)
                    for mdi in self.metadata_intervals]
                data = numpy.array(ebasmeta, dtype=NETCDFSTRING)
            var[data_ind] = data

    def add_variables(self):
        """
        Add the time variables and the time dimenstion.
        """
        self.gen_cdm_variables()
        self.dims = set()
        for cdm_var in self.cdm_variables.values():
            self.add_var_dimensions(cdm_var)
            if cdm_var['var_type'] == VAL:
                # only for data variable: add additionsl variable dimensions
                # add the variable for holding the measurements
                var = self.ncfile.createVariable(
                    cdm_var['cdm_varname'], numpy.float64,
                    cdm_var['dimensions']+('time', ),
                    fill_value=numpy.float64(numpy.nan))
            if cdm_var['var_type'] == MTA:
                # add the variable for holding the variable metadata
                var = self.ncfile.createVariable(
                    cdm_var['cdm_varname'], NETCDFSTRING,
                    cdm_var['dimensions']+('metadata_time', ))
            if cdm_var['var_type'] == FLG:
                # add the flag dimension for this variable:
                dimname = cdm_var['cdm_varname'] + '_flags'
                self.ncfile.createDimension(dimname, cdm_var['flag_dimsize'])
                #var = self.ncfile.createVariable(dimname, numpy.int32,
                #                                 (dimname, ))
                # add the variable for holding the flags
                var = self.ncfile.createVariable(
                    cdm_var['cdm_varname'], numpy.int32,
                    cdm_var['dimensions']+(dimname, 'time'),
                    fill_value=numpy.int32(0))

            self.add_var_attributes(var, cdm_var)
            self.add_var_data(var, cdm_var)



"""
$Id: qm_qv.py 2466 2020-07-10 13:20:48Z pe $

EBAS Masterdata Class for quality measure and -validity masterdata

This module implements the classes EbasMasterQM and EbasMasterQV.

History:
V.1.1.0  2016-10-17  pe  initial version

"""


from __future__ import absolute_import
# otherwise re is imported relative as masterdata.re

import re
import six
from .offline_masterdata import QMOfflineMasterData, QVOfflineMasterData, \
    QOOfflineMasterData
from .base import EbasMasterBase

class EbasMasterQM(EbasMasterBase, QMOfflineMasterData):
    """
    Domain Class for quality measure masterdata
    Objects of this class do not represent entities, this class provides class
    methods for handling components and checking them against master data.
    Components master data are retrieved from database or from offline storage
    when no database access is possible.
    """

    # read offline masterdata for QM (quality measure)
    # Those are fallback values, will be read from database as soon as possible.
    QMOfflineMasterData.read_pickle_file()

    def __init__(self, dbh=None):
        """
        Read qm masterdata if dbh is provided.
        """
        QMOfflineMasterData.__init__(self)
        if not self.__class__.INIT and dbh:
            self.from_db(dbh)

    def new_reference_calibration(self, name):
        """
        Check if the qa measure ID is generic (O#) reference calibration
        which must be generated on import.
        Parameters:
            mane    QM ID
        Returns:
            (or_code, date) if it is a new reference calibration
            None if it is no new reference calibration
        """
        try:
            self[name]
        except KeyError:
            if isinstance(name, six.string_types):
                reg = re.match(
                    r'(.....)_reference_calibration_(\d\d\d\d\d\d\d\d)',
                    name)
                if reg:
                    return (reg.group(1), reg.group(2))
        return None




class EbasMasterQV(QVOfflineMasterData):
    """
    Domain Class for qualitymeasure validity masterdata
    Objects of this class do not represent entities, this class provides class
    methods for handling components and checking them against master data.
    Components master data are retrieved from database or from offline storage
    when no database access is possible.
    """

    # read offline masterdata for QV (qualitymeasure validity)
    # Those are fallback values, will be read from database as soon as possible.
    QVOfflineMasterData.read_pickle_file()
    # some hardcoded exceptional parameters (see method exceptional):
    #QV_EXCEPTIONAL = EbasMasterQV._qv_exceptions()

    def __getitem__(self, key):
        """
        Allows dictionary like access to metadata.
        Exception for QV:
            - exceptional metadata lookup (non DB, data level dependent)
        Parameters:
            key    (qm_id, ft_type, comp_name, data_level)
                   Only the first 3 are used for regular lookup!
                   data_level is optional and is used for finding exceptional
                   data level dependent masterdata
        """
        # first try: use regular masterdata, ignore data level
        try:
            return self.__class__.META[key[:3]]
        except KeyError:
            # don't exist: try exceptional masterdata (needs data level)
            # Those masterdata are NOT defined in the database, but might be
            # used e.g. in lev 0 files.
            # Thus they are accepted when reading the file, but the domain
            # layer will issue an error message.
            if len(key) != 4:
                raise
            data_level = key[3]
            if data_level in ('0a', '0b'):
                data_level = '0'
            return self.__class__.QV_EXCEPTIONAL[data_level][key[:3]]

    def __init__(self, dbh=None):
        """
        Read qv masterdata if dbh is provided.
        """
        QVOfflineMasterData.__init__(self)
        if not self.__class__.INIT and dbh:
            self.from_db(dbh)
        # recalculate exceptional masterdata after db read:
        self.__class__.QV_EXCEPTIONAL = self._qv_exceptions()

    def list(self, qm_id=None, ft_type=None, co_comp_name=None):
        """
        Generates a list of matching masterdata.
        Parameters:
            qm_id             quality measure ID
            ft_type           fieldinstrument type
            co_comp_name      component name
        Returns:
            generator of QV dictionaries with matching criteria
        """
        for qv_ in list(self.__class__.META.values()):
            if (qm_id is None or qv_['QM_ID'] == qm_id) and \
               (ft_type is None or qv_['FT_TYPE'] == ft_type) and \
               (co_comp_name is None or qv_['CO_COMP_NAME'] == co_comp_name):
                yield qv_

    @classmethod
    def _qv_exceptions(cls):
        """
        Generate dictionary for exceptions (special datalevels, not imported but
        should be just file format checked.
        Parameters:
            None
        Returns:
            exception dict
        """
        exceptions = {'0': {}, '1': {}}

        # NOxy lev0: same QM_ID's as defined for NO, NO2 and NOx
        qm_list = list(set([qv_['QM_ID'] for qv_ in list(cls.META.values())
                            if qv_['FT_TYPE'] in
                            ('chemiluminescence_photolytic',
                             'chemiluminescence_molybdenum',
                             'chemiluminescence_photometer') and
                            qv_['CO_COMP_NAME'] in
                            ('nitrogen_monoxide', 'nitrogen_dioxide',
                             'NOx')]))
        basedict = {
        }
        for qm_id in qm_list:
            for inst in ('chemiluminescence_molybdenum',
                         'chemiluminescence_photolytic',
                         'chemiluminescence_photometer'):
                for comp in ('NO_#counts', 'NO_converter_#counts',
                             'NO_sensitivity', 'converter_efficiency',
                             'temperature', 'pressure', 'status'):
                    dict_ = basedict.copy()
                    dict_.update({'CO_COMP_NAME': comp, 'QM_ID': qm_id,
                                  'FT_TYPE': inst})
                    exceptions['0'][(dict_['QM_ID'], dict_['FT_TYPE'],
                                     dict_['CO_COMP_NAME'])] = dict_
        # NOxy lev1: same QM_ID's as defined for NO, NO2 and NOx
        qm_list = list(set([qv_['QM_ID'] for qv_ in list(cls.META.values())
                            if qv_['FT_TYPE'] in
                            ('chemiluminescence_photolytic',
                             'chemiluminescence_molybdenum',
                             'chemiluminescence_photometer') and
                            qv_['CO_COMP_NAME'] in
                            ('nitrogen_monoxide', 'nitrogen_dioxide',
                             'NOx')]))
        basedict = {
        }
        for qm_id in qm_list:
            for inst in ('chemiluminescence_molybdenum',
                         'chemiluminescence_photolytic',
                         'chemiluminescence_photometer'):
                for comp in ('temperature', 'pressure'):
                    dict_ = basedict.copy()
                    dict_.update({'CO_COMP_NAME': comp, 'QM_ID': qm_id,
                                  'FT_TYPE': inst})
                    exceptions['1'][(dict_['QM_ID'], dict_['FT_TYPE'],
                                     dict_['CO_COMP_NAME'])] = dict_
        # VOC (online_gc)
        for inst in ('online_gc',):
            qv_list = list(set([(qv_['QM_ID'], qv_['CO_COMP_NAME'])
                                for qv_ in list(cls.META.values())
                                if qv_['FT_TYPE'] == inst]))
            for qv_ in qv_list:
                basedict = {
                    'QM_ID': qv_[0],
                    'FT_TYPE': inst,
                }
                for ext in ('_peak_area', '_peak_width', '_retention_time'):
                    dict_ = basedict.copy()
                    dict_.update({'CO_COMP_NAME': qv_[1] + ext})
                    exceptions['0'][(dict_['QM_ID'], dict_['FT_TYPE'],
                                     dict_['CO_COMP_NAME'])] = dict_
            # distinct QM_ID:
            qm_list = list(set([x[0] for x in qv_list]))
            for qm_ in qm_list:
                for comp in ('status',):
                    exceptions['0'][(qm_, inst, comp)] = {
                        'QM_ID': qm_,
                        'FT_TYPE': inst,
                        'CO_COMP_NAME': comp}

        # filter_absorption_photometer (magee AE31, AE33) lev 0:
        # allow the same QM_ID's as defined for filter absorption_photometer
        qm_list = list(set([qv_['QM_ID'] for qv_ in list(cls.META.values())
                            if qv_['FT_TYPE'] == 'filter_absorption_photometer'
                            and qv_['CO_COMP_NAME'] ==
                            'aerosol_absorption_coefficient']))
        basedict = {
            'FT_TYPE': 'filter_absorption_photometer',
        }
        for qm_id in qm_list:
            # for each of those QM_IDs allow all components from AE31 and AE33
            # lev0 template
            for comp in ('flow_rate', 'status',
                         'equivalent_black_carbon', 'relative_humidity',
                         'filter_number', 'biomass_burning_aerosol_fraction',
                         'reference_beam_signal', 'sensing_beam_signal',
                         'filter_loading_compensation_parameter',
                         'equivalent_black_carbon_loading',
                         'attenuation_coefficient', 'sensing_zero_signal',
                         'reference_zero_signal', 'bypass_fraction',
                         'transmittance', 'sample_intensity',
                         'reference_intensity', 'sample_length_on_filter'):
                dict_ = basedict.copy()
                dict_.update({'CO_COMP_NAME': comp, 'QM_ID': qm_id})
                exceptions['0'][(dict_['QM_ID'], dict_['FT_TYPE'],
                                 dict_['CO_COMP_NAME'])] = dict_

        # nephelometer lev 0:
        # allow the same QM_ID's as defined for nephelometer
        qm_list = list(set([qv_['QM_ID'] for qv_ in list(cls.META.values())
                            if qv_['FT_TYPE'] == 'nephelometer' and
                            qv_['CO_COMP_NAME'] in
                            ('aerosol_light_backscattering_coefficient',
                             'aerosol_light_scattering_coefficient')]))
        basedict = {
            'FT_TYPE': 'nephelometer',
        }
        for qm_id in qm_list:
            for comp in (
                    'flow_rate', 'electric_tension', 'electric_current',
                    'status',
                    'aerosol_light_scattering_coefficient_zero_measurement',
                    'aerosol_light_backscattering_coefficient_zero_measurement',
                    'aerosol_light_rayleighscattering_coefficient_zero_measurement'):
                dict_ = basedict.copy()
                dict_.update({'CO_COMP_NAME': comp, 'QM_ID': qm_id})
                exceptions['0'][(dict_['QM_ID'], dict_['FT_TYPE'],
                                 dict_['CO_COMP_NAME'])] = dict_

        # dmps lev 0:
        # allow the same QM_ID's as defined for dmps
        qm_list = list(set([(qv_['FT_TYPE'], qv_['QM_ID'])
                            for qv_ in list(cls.META.values())
                            if qv_['FT_TYPE'].endswith('mps') and
                                qv_['CO_COMP_NAME'] ==
                                    'particle_number_size_distribution']))
        for ft_type, qm_id in qm_list:
            for comp in ('flow_rate', 'status', 'particle_diameter',
                         'particle_number_concentration'):
                dict_ = {
                    'FT_TYPE': ft_type,
                    'QM_ID': qm_id,
                    'CO_COMP_NAME': comp,
                    }
                exceptions['0'][(dict_['QM_ID'], dict_['FT_TYPE'],
                                 dict_['CO_COMP_NAME'])] = dict_

        # CCNC, DMPS-CCNC lev 0:
        # allow the same QM_ID's as defined for CCNC
        qm_list = list(set([qv_['QM_ID'] for qv_ in list(cls.META.values())
                            if qv_['FT_TYPE'] == 'CCNC' and
                            qv_['CO_COMP_NAME'] in
                            ('cloud_condensation_nuclei_number_concentration',)]))
        for qm_id in qm_list:
            for comp in ('supersaturation', 'electric_tension',
                         'electric_current', 'status', 'flow_rate',
                         'temperature_gradient'):
                for inst in ('CCNC', 'DMPS-CCNC'):
                    dict_ = {
                        'FT_TYPE': inst,
                        'CO_COMP_NAME': comp,
                        'QM_ID': qm_id,
                        }
                    exceptions['0'][(dict_['QM_ID'], dict_['FT_TYPE'],
                                 dict_['CO_COMP_NAME'])] = dict_
            # only DMPS-CCNC
            for comp in ('particle_diameter',
                         'cloud_condensation_nuclei_number_concentration'):
                    dict_ = {
                        'FT_TYPE': 'DMPS-CCNC',
                        'CO_COMP_NAME': comp,
                        'QM_ID': qm_id,
                        }
                    exceptions['0'][(dict_['QM_ID'], dict_['FT_TYPE'],
                                 dict_['CO_COMP_NAME'])] = dict_

        # cpc lev0
        # allow the same QM_ID's as defined for cpc
        qm_list = list(set([qv_['QM_ID'] for qv_ in list(cls.META.values())
                            if qv_['FT_TYPE'] == 'cpc' and
                            qv_['CO_COMP_NAME'] in
                            ('particle_number_concentration',)]))
        for qm_id in qm_list:
            for comp in ('pulse_width', 'flow_rate', 'relative_humidity'):
                dict_ = {
                    'FT_TYPE': 'cpc',
                    'CO_COMP_NAME': comp,
                    'QM_ID': qm_id,
                    }
                exceptions['0'][(dict_['QM_ID'], dict_['FT_TYPE'],
                             dict_['CO_COMP_NAME'])] = dict_

        return exceptions


class EbasMasterQO(QOOfflineMasterData):
    """
    Domain Class for quality assurance outcome masterdata
    Objects of this class do not represent entities, this class provides class
    methods for handling components and checking them against master data.
    Components master data are retrieved from database or from offline storage
    when no database access is possible.
    """

    # read offline masterdata for QM (quality measure)
    # Those are fallback values, will be read from database as soon as possible.
    QOOfflineMasterData.read_pickle_file()

    def __getitem__(self, name):
        """
        Allows dictionary like access to metadata
        """
        return self.__class__.META[name]

    def __init__(self, dbh=None):
        """
        Read qm masterdata if dbh is provided.
        """
        QOOfflineMasterData.__init__(self)
        if not self.__class__.INIT and dbh:
            self.from_db(dbh)



"""
Csv Unicode file format module
$Id: csv_unicode.py 2715 2021-10-13 14:25:10Z pe $
"""
import csv, codecs
from six import PY2, StringIO

class UTF8Recoder(object):
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    ONLY used for PY2
    """
    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def __next__(self):
        return self.reader.next().encode("utf-8")

    def next(self):
        """
        For python2
        """
        return self.__next__()

class CsvUnicodeReader(object):
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        if PY2:
            f = UTF8Recoder(f, encoding)
            self.reader = csv.reader(f, dialect=dialect, **kwds)
        else:
            self.reader = csv.reader(codecs.getreader(encoding)(f),
                                     dialect=dialect, **kwds)

    def __next__(self):
        row = next(self.reader)
        if PY2:
            return [unicode(s, "utf-8") for s in row]
        return row

    def next(self):
        """
        For python2
        """
        return self.__next__()

    def __iter__(self):
        return self

class CsvUnicodeWriter:
    """
    A CSV writer which will write rows to CSV file "f",
    which is encoded in the given encoding.

    For PY2, all output streams were raw, so we had to encode all streams the
    same way.
    For PY3, the file object might encode already, and csv.writer may be called
    with a PY3 str (which is unicode). Nevertheless, in PY 3 one can also open
    a file in "wb" mode, which requires the same treatment as in PY2. In this
    case, use rawfile=False!
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", rawfile=True,
                 **kwds):
        """
        Set up the object
        """
        self.rawfile = rawfile
        # Redirect output to a queue
        self.queue = StringIO()
        if rawfile:
            self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
        else:
            self.writer = csv.writer(f, dialect=dialect, **kwds)
        self.stream = f
        self.encoder = codecs.getincrementalencoder(encoding)()

    def writerow(self, row):
        if self.rawfile:
            if PY2:
                self.writer.writerow([unicode(s).encode("utf-8") for s in row])
                # Fetch UTF-8 output from the queue ...
                data = self.queue.getvalue()
                data = data.decode("utf-8")
            else:
                self.writer.writerow([str(s) for s in row])
                # Fetch UTF-8 output from the queue ...
                data = self.queue.getvalue()
            # ... and reencode it into the target encoding
            data = self.encoder.encode(data)
            # write to the target stream
            self.stream.write(data)
            # empty queue
            self.queue.truncate(0)
        else:
            self.writer.writerow([s if isinstance(s, str) else str(s) for s in row])

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)



"""
/NasaAmes1001/nasa_ames.py
$Id: base.py 2582 2020-12-09 19:23:48Z pe $

Base class for NASA Ames 1001.

History:
V.1.0.0  2013-03-05  pe  initial version

"""
import logging
from nilutility.float_helper import float_almostequal
from nilutility.datatypes import DataObject

class NasaAmes1001Error(Exception):
    """
    Error class for NasaAmes 1001.
    """
    pass

class NasaAmes1001Base(object):
    # pylint: disable=R0902
    # R0902: Too many instance attributes
    """
    Partial class NasaAmes1001: Base class used by Parts.
    """
    # all data attibutes (set on read, getattr implemented)
    DATA_ATTRIBS = [
        'NLHEAD',   # int, nuber of header lines
        'ONAME',    # str, free text for 1001
        'ORG',      # str, free text for 1001
        'SNAME',    # str, free text for 1001
        'MNAME',    # str, free text for 1001
        'IVOL',     # int
        'NVOL',     # int
        'DATE',     # datetime.datetime object
        'RDATE',    # datetime.datetime object
        'DX',       # float
        'XNAME',    # str, free text for 1001
        'NV',       # int, number of dependent variables in file
        'VSCAL',    # list of float, len=NV, scaling factors for data
        'VMISS',    # list of str, len=NV, missing values used for data
        'VFORMAT',  # list of str, len=NV, format str. (gen. from VMISS)
        'VREPR',    # list of type, internal representation (datatype)
        'VNAME',    # str, free text for 1001
        'SCOML',    # list of str, free texts
        'NCOML',    # list of str, free texts for 1001
        'DATA',     # list of lists of float/Decimal
        ]

    # attributes that can be set (before write), others will be calculated
    SETABLE_DATA_ATTRIBS = ['ONAME', 'ORG', 'SNAME', 'MNAME', 'IVOL', 'NVOL',
                            'DATE', 'RDATE', 'XNAME', 'VSCAL', 'VNAME', 'SCOML',
                            'NCOML', 'DATA']
    # read/write states
    SECTIONS = ['NLHEAD', 'ONAME', 'ORG', 'SNAME', 'MNAME', 'IVOL_NVOL',
                'DATE_RDATE', 'DX', 'XNAME', 'NV', 'VSCAL', 'VMISS', 'VNAME',
                'SCOML', 'NCOML', 'DATA']

    # for bitfield strictness:
    # strictly require data in format given by VMISS
    STRICT_NUMFORMAT = 1
    STRICT_DX = 2
    # strictly require irregular data when DX=0 (DX!=0 will always raise an
    # error if not obeyed in data)

    def __setattr__(self, name, value):
        if name in self.__class__.SETABLE_DATA_ATTRIBS:
            self.data[name] = value
        else:
            object.__setattr__(self, name, value)

    def __getattr__(self, name):
        try:
            return self.data[name]
        except KeyError as expt:
            raise AttributeError(expt)

    def __init__(self, strictness=0):
        """
        Paramaters:
            strictness   bit field for strictness on read
                         (STRICT_NUMFORMAT, STRICT_DX)
        Returns:
            None
        Raises:
            IOError on file open problems
            NasaAmesError
        """
        self.file = None
        self.encoding = None
        self.enc_line = None
        self.state = -1
        self.errors = 0
        self.warnings = 0
        self.strictness = strictness
        self.lnum = 0
        # some special attributes:
        # xformat: calcuated and used internally by write:
        self.xformat = None
        # xmindig: minimum digit of x used in the physical file
        # (order of magnitude)
        # is set by the reader when reading the data lines
        self.xmindig = None
        # min/max xmindig: can be set by the caller to restrict the output
        # precision of x
        # eg. minxmindig =-4, maxxmindig=-1: variable will have at least 1
        # digit after comma, but not more than 4
        self.minxmindig = None
        self.maxxmindig = None

        # min/max vmindig: minimum digit to be used for each var (order of
        # magnitude) can be set by the caller to restrict the precision
        # eg. minvmindig =-4, maxvmindig=-1: variable will have at least 1
        # digit after comma, but not more than 4
        self.minvmindig = []
        self.maxvmindig = []

        # self.minvmiss: caller can request to generate a minimal VMISS value
        # for the respective variable (do not add one order of magnitude if
        # the max value is less than the calculated VMISS)
        # Set to True if wanted for a given variable
        self.minvmiss = []

        # self.vmiss: caller can request a specifiv VMISS value.
        # This is only used if it can be used according to data (type and
        # maxvalue), minvmindig and maxvmindig
        self.vmiss = []

        # the main data attribute, which represents the file's content
        self.data = None
        self.init_data()

        self.logger = logging.getLogger("NasaAmes1001")

    def init_data(self):
        """
        Initializes the data object. Sets all attributes to None.
        Parameters:
            None
        Returns:
            None
        """
        self.data = DataObject(dict([(att, None)
                                     for att in self.__class__.DATA_ATTRIBS]))

    def init_state(self):
        """
        Initialize the object state before reading or writing operations.
        Parameters:
            None
        Returns:
            None
        """
        self.file = None
        self.state = -1
        self.errors = 0
        self.warnings = 0
        self.lnum = 0

    def curr_state_name(self):
        """
        Returns the name of the current reading state.
        Parameters:
            None
        Returns:
            state name (str)
        """
        return self.__class__.SECTIONS[self.state]

    def dx_from_data(self):
        """
        Calculates DX from the sample intervals.
        Parameters:
            None
        Returns:
            DX (real)
        There are some problematic cases with float representation:
            e.g. the sequence 1.1, 1.2, 1.3, ...
            would result in a dx between 0.09999999999999987
            and 0.10000000000000009
            (even worse with e.g. 1.000000001, 1.000000002, ...)

            Using Decimal instead of float would raise performance issues

            Solution: check dx with float operations (and almostequal)
            Then, in the end calculate a Decimal dx using the first two samples
            only.
        """
        # calculate dx
        if not self.data.DATA or not self.DATA[0] or len(self.DATA[0]) < 2:
            return 0
        if None in self.DATA[0]:
            raise NasaAmes1001Error(
                "missing values not allowed for x-values")
        if self.xmindig:
            # we need to round the values to xmindig
            magnitude = self.xmindig
            if magnitude > 0:
                magnitude -= 1
            magnitude *= -1
            data = [round(x, magnitude) for x in self.DATA[0]]
            # We test the average dx of all values (maybe rounded)
            # the result must be rounded to mindig.
            # In py3, we have additionally an issue when subtracting floats due
            # to number representation:
            # e.g. 309.625 - 309.583333 = 0.041667000000018106
            #      0.625-0.583333 = 0.04166700000000001
            # So rounding is even more crucial in python3.
            dx_ = round(
                sum([data[i]-data[i-1] for i in range(1, len(data))]) / \
                    (len(data)-1),
                magnitude)
        else:
            data = self.DATA[0]
            # We test the average dx of all values (unrounded, no xmindig)
            # I guess it will never be called w/o xmindig anyway
            dx_ = sum([data[i]-data[i-1] for i in range(1, len(data))]) / \
                (len(data)-1)
        for diff in zip(data, data[1:]):
            if not self._dx_equal(float(diff[1]-diff[0]), dx_):
                return 0
        return dx_

    def _dx_equal(self, dx1, dx2):
        """
        Compares two dx values, allowing differences according to the objects
        xmindig.
        Parameters:
            dx1, dx2    the two dx values to be compared
        Returns:
            True/False
        """
        # if x output precission is restricted by the caller, use this
        # precission as comp. criteria
        if self.xmindig:
            magnitude = self.xmindig
            if self.xmindig >= 1:
                # if magnitude is at least whole integers, we just compare
                # we had problems with monthly data (29-31 days), which
                # generated dx=30, because of allowing +- 1 in dx comparison,
                # which is wrong.
                return dx1 == dx2
            if abs(dx1 - dx2) >= 1.1 * 10 ** magnitude:
                # If difference is bigger than one digit difference in the
                # lowest represented digit, DX is different
                # This accommodates rounding issues in the last digit.
                return False
        else:
            return float_almostequal(dx1, dx2)
        return True

    def error(self, msg, lnum=True, exception=False, lnum_override=None,
              counter_increase=1):
        """
        Error handling for file read.
        Error will be logged, lnum included if passed.
        Parameters:
            msg         error message
            lnum        include line number (optional)
            exception   raises exception if true
            lnum_override
                        override the internal line number
        Returns:
            None
        Raises:
            NasaAmesError when parameter exception is True
        """
        self.errors += counter_increase
        if lnum:
            msg = "line {}: {}".format(
                lnum_override if lnum_override is not None else self.lnum, msg)
        self.logger.error(msg)
        if exception:
            raise NasaAmes1001Error(msg)

    def warning(self, msg, lnum=True, lnum_override=None, counter_increase=1):
        """
        Warning message for file read.
        Worning will be logged, lnum included if passed.
        Parameters:
            msg         warning message
            lnum        include line number (optional)
        Returns:
            None
        """
        self.warnings += counter_increase
        if lnum:
            msg = "line {}: {}".format(
                lnum_override if lnum_override is not None else self.lnum, msg)
        self.logger.warning(msg)



"""
$Id: indexdb.py 2792 2022-03-18 13:15:32Z pe $

DB layer for fileindex db.
The fileindex IndexDB is a sqlite3 DB used to provide a basic metadata database
which can accompany a file archive (extracted from ebas).
"""
from textwrap import dedent
from databases.sqlite import SqliteDb


class IndexDb(SqliteDb):
    """
    EBAS fileindex DB I/O class.
    """

    def __init__(self, filename):
        """
        Initialize DB connection. Open DB, create tables.
        Parameters:
            filename   filename for the database. If prepended with '+', ignore
                       errors while creating tables.
        """
        super(IndexDb, self).__init__(filename)
        self._stationcodes = None

    def insert_file_obj(self, ebasfile):
        """
        Main trigger method: perform all inserts reated to the data file.
        Parameters:
            ebasfile   ebas.io.file.EbasFile object
        Returns:
            None
        """
        station_meta = (
            'station_code', 'platform_code', 'station_name',
            'station_wdca_id', 'station_gaw_name', 'station_gaw_id',
            'station_airs_id', 'station_other_ids', 'station_state_code',
            'station_landuse', 'station_setting', 'station_gaw_type',
            'station_wmo_region', 'station_latitude', 'station_longitude',
            'station_altitude')
        station_dict = {key: ebasfile.metadata[key] for key in station_meta}
        self.insert_station(station_dict)

        var_meta = ('station_code', 'matrix', 'comp_name', 'unit', 'statistics',
                    'instr_type', #'instr_ref',
                    'method',
                    'revdate', 'period', 'resolution', 'datalevel',
                    'filename')
        for i, var in enumerate(ebasfile.variables):
            var_dict = {key: ebasfile.get_meta_for_var(i, key)
                        for key in var_meta}
            var_dict['vnum'] = i
            var_dict['instr_ref'] = ebasfile.get_meta_for_var(i, 'lab_code') + \
                '_' + ebasfile.get_meta_for_var(i, 'instr_name')
            var_dict['first_start'] = ebasfile.sample_times[0][0]
            var_dict['first_end'] = ebasfile.sample_times[0][1]
            var_dict['last_start'] = ebasfile.sample_times[-1][0]
            var_dict['last_end'] = ebasfile.sample_times[-1][1]
            var_id = self.insert_variable(var_dict)
            for cha in var.metadata.characteristics:
                cha_dict = {
                    'var_id': var_id,
                    'ct_type': cha.CT_TYPE,
                    'datatype': cha.CT_DATATYPE,
                    }
                if cha.CT_DATATYPE == 'INT':
                    cha_dict['val_int'] = cha.DC_VAL_INT
                elif cha.CT_DATATYPE == 'DBL':
                    cha_dict['val_dbl'] = cha.DC_VAL_DBL
                else:
                    cha_dict['val_chr'] = cha.DC_VAL_CHR
                self.insert_characteristic(cha_dict)

    def create_tables(self):
        """
        Create all necessary tables.
        Parameters:
            None
        Returns:
            None
        Raises:
            IndexDbExists  when the tables already exist
        """
        self.create_table_station()
        self.create_table_variable()
        self.create_table_characteristic()
        self.dbh.commit()
        self.logger.debug("tables initialized")

    def create_table_station(self):
        """
        Create station table.
        """
        curs = self.dbh.cursor()
        curs.execute(dedent("""\
            create table station
            (
               station_code TEXT not NULL primary key,
               platform_code TEXT not NULL,
               station_name TEXT,
               station_wdca_id TEXT,
               station_gaw_name TEXT,
               station_gaw_id TEXT,
               station_airs_id TEXT,
               station_other_ids TEXT,
               station_state_code TEXT,
               station_landuse TEXT,
               station_setting TEXT,
               station_gaw_type TEXT,
               station_wmo_region INTEGER,
               station_latitude REAL,
               station_longitude REAL,
               station_altitude REAL)
            """))

    def create_table_variable(self):
        """
        Create station table.
        """
        curs = self.dbh.cursor()
        curs.execute(dedent("""\
            create table variable
            (
               var_id integer not null primary key,
               station_code TEXT not NULL references station (station_code),
               matrix TEXT not NULL,
               comp_name TEXT not NULL,
               unit TEXT not NULL,
               statistics TEXT not NULL,
               instr_type TEXT not NULL,
               instr_ref TEXT not NULL,
               method TEXT not NULL,
               first_start TEXT not NULL,
               first_end TEXT not NULL,
               last_start TEXT not NULL,
               last_end TEXT not NULL,
               revdate TEXT not NULL,
               period TEXT not NULL,
               resolution TEXT not NULL,
               datalevel TEXT,
               filename TEXT not NULL,
               vnum integer not null)
            """))

    def create_table_characteristic(self):
        """
        Create station table.
        """
        curs = self.dbh.cursor()
        curs.execute(dedent("""\
            create table characteristic
            (
               var_id integer not null references variable(var_id),
               ct_type text not null,
               datatype text not null,
               val_int integer,
               val_dbl float,
               val_chr text)
            """))

    @property
    def stationcodes(self):
        """
        Property for selecting a station code list.
        Parameters:
            None
        Returns:
            Station code list from table station.
        """
        if self._stationcodes is None:
            curs = self.dbh.cursor()
            curs.execute("select station_code from station")
            self._stationcodes = [x[0] for x in curs.fetchall()]
        return self._stationcodes

    def insert_station(self, station_dict):
        """
        Insert station metadata into the IndexDb.
        """
        if station_dict['station_code'] not in self.stationcodes:
            curs = self.dbh.cursor()
            curs.execute(dedent("""\
                insert into station
                   (station_code, platform_code, station_name,
                    station_wdca_id, station_gaw_name, station_gaw_id,
                    station_airs_id, station_other_ids, station_state_code,
                    station_landuse, station_setting, station_gaw_type,
                    station_wmo_region, station_latitude, station_longitude,
                    station_altitude)
                values
                   (:station_code, :platform_code, :station_name,
                    :station_wdca_id, :station_gaw_name, :station_gaw_id,
                    :station_airs_id, :station_other_ids, :station_state_code,
                    :station_landuse, :station_setting, :station_gaw_type,
                    :station_wmo_region, :station_latitude, :station_longitude,
                    :station_altitude)
                """), station_dict)
            self.dbh.commit()
            self._stationcodes.append(station_dict['station_code'])
            self.logger.debug("insert station %s", station_dict['station_code'])

    def insert_variable(self, var_dict):
        """
        Insert metadata for one variable of one file into the IndexDb.
        """
        curs = self.dbh.cursor()
        curs.execute(dedent("""\
            insert into variable
               (vnum, station_code, matrix, comp_name, unit, statistics,
                instr_type, instr_ref, method,
                first_start, first_end, last_start, last_end,
                revdate, period, resolution, datalevel,
                filename)
            values
               (:vnum, :station_code, :matrix, :comp_name, :unit, :statistics,
                :instr_type, :instr_ref, :method,
                :first_start, :first_end, :last_start, :last_end,
                :revdate, :period, :resolution, :datalevel,
                :filename)
            """), var_dict)
        var_id = curs.lastrowid
        self.dbh.commit()
        self.logger.debug("insert variable %s. var_id=%d", var_dict, var_id)
        return var_id

    def insert_characteristic(self, cha_dict):
        """
        Insert metadata for one characteristic of one variable of one file into
        the IndexDb.
        """
        curs = self.dbh.cursor()
        curs.execute(dedent("""\
            insert into characteristic
               (var_id, ct_type, datatype, val_{0})
            values
               (:var_id, :ct_type, :datatype, :val_{0})
            """.format(cha_dict['datatype'].lower())), cha_dict)
        self.dbh.commit()
        self.logger.debug("insert characteristic for variable {}", cha_dict)



"""
$Id: org.py 2088 2018-06-29 13:46:38Z pe $

EBAS Masterdata Class for organization masterdata

Remark: or is a keyword in python, cannot be used as identifier (thus not as
module name, otherwise imports are syntactically wrong). Therefore this module
is named org, not or as it would be following the naming convention).

This module implements the class EbasMasterOR.

History:
V.1.0.0  2016-05-25  pe  initial version

"""

from .offline_masterdata import OROfflineMasterData
from .base import EbasMasterBase

class EbasMasterOR(EbasMasterBase, OROfflineMasterData):
    """
    Domain Class for organization masterdata.
    Objects of this class do not represent entities, this class provides class
    methods for handling organizations and checking them against master data.
    Organization master data are retrieved from database or from offline
    storage when no database access is possible.
    """

    # read offline masterdata for OR (organizations)
    # Those are fallback values, will be read from database as soon as possible.
    OROfflineMasterData.read_pickle_file()

    def __init__(self, dbh=None):
        """
        Read or masterdata if dbh is provided.
        """
        OROfflineMasterData.__init__(self)
        if not self.__class__.INIT and dbh:
            self.from_db(dbh)
